@@ -2,8 +2,8 @@
  *      classbuilder.c - this file is part of Geany, a fast and lightweight IDE
  *
  *      Copyright 2007 Alexander Rodin <rodin(dot)alexander(at)gmail(dot)com>
- *      Copyright 2007-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: classbuilder.c 2430 2008-03-31 13:09:56Z ntrel $
+ * $Id: classbuilder.c 3446 2009-01-04 18:30:42Z eht16 $
  */
 
 /* Class Builder - creates source files containing a new class interface and definition. */
@@ -30,19 +30,24 @@
 #include "support.h"
 #include "filetypes.h"
 #include "document.h"
-#include "pluginmacros.h"
+#include "editor.h"
+#include "ui_utils.h"
+#include "geanyfunctions.h"
 
 
-PluginFields	*plugin_fields;
 GeanyData		*geany_data;
+GeanyFunctions	*geany_functions;
 
 
-PLUGIN_VERSION_CHECK(7)
+PLUGIN_VERSION_CHECK(GEANY_API_VERSION)
 
-PLUGIN_INFO(_("Class Builder"), _("Creates source files for new class types."), VERSION,
+PLUGIN_SET_INFO(_("Class Builder"), _("Creates source files for new class types."), VERSION,
 	"Alexander Rodin")
 
 
+static GtkWidget *main_menu_item = NULL;
+
+
 enum
 {
 	GEANY_CLASS_TYPE_CPP,
@@ -120,16 +125,16 @@ G_BEGIN_DECLS\n\
 \n\
 #define {class_name_up}_TYPE				({class_name_low}_get_type())\n\
 #define {class_name_up}(obj)				(G_TYPE_CHECK_INSTANCE_CAST((obj),\\\n\
-		{class_name_up}_TYPE, {class_name}))\n\
+			{class_name_up}_TYPE, {class_name}))\n\
 #define {class_name_up}_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST((klass),\\\n\
-		{class_name_up}_TYPE, {class_name}Class))\n\
-#define IS_{class_name_up}(obj)				(G_TYPE_CHECK_INSTANCE_TYPE((obj),\\\n\
-		{class_name_up}_TYPE))\n\
-#define IS_{class_name_up}_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE((klass),\\\n\
-		{class_name_up}_TYPE))\n\
+			{class_name_up}_TYPE, {class_name}Class))\n\
+#define IS_{class_name_up}(obj)			(G_TYPE_CHECK_INSTANCE_TYPE((obj),\\\n\
+			{class_name_up}_TYPE))\n\
+#define IS_{class_name_up}_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE((klass),\\\n\
+			{class_name_up}_TYPE))\n\
 \n\
-typedef struct _{class_name}				{class_name};\n\
-typedef struct _{class_name}Class			{class_name}Class;\n\
+typedef struct _{class_name}			{class_name};\n\
+typedef struct _{class_name}Class		{class_name}Class;\n\
 \n\
 struct _{class_name}\n\
 {\n\
@@ -142,7 +147,7 @@ struct _{class_name}Class\n\
 	{base_name}Class parent_class;\n\
 };\n\
 \n\
-GType		{class_name_low}_get_type	(void);\n\
+GType		{class_name_low}_get_type		(void);\n\
 {constructor_decl}\
 \n\
 G_END_DECLS\n\
@@ -153,10 +158,10 @@ G_END_DECLS\n\
 static const gchar templates_gtk_class_source[] = "{fileheader}\n\
 #include \"{header}\"\n\
 \n\
-typedef struct _{class_name}Private				{class_name}Private;\n\
+typedef struct _{class_name}Private			{class_name}Private;\n\
 \n\
 #define {class_name_up}_GET_PRIVATE(obj)		(G_TYPE_INSTANCE_GET_PRIVATE((obj),\\\n\
-		{class_name_up}_TYPE, {class_name}Private))\n\
+			{class_name_up}_TYPE, {class_name}Private))\n\
 \n\
 struct _{class_name}Private\n\
 {\n\
@@ -164,7 +169,7 @@ struct _{class_name}Private\n\
 };\n\
 \n\
 static void {class_name_low}_class_init			({class_name}Class *klass);\n\
-static void {class_name_low}_init				({class_name} *self);\n\
+static void {class_name_low}_init      			({class_name} *self);\n\
 {destructor_decl}\
 \n\
 /* Local data */\n\
@@ -189,19 +194,19 @@ GType {class_name_low}_get_type(void)\n\
 			NULL /* value_table */\n\
 		};\n\
 		\n\
-		self_type = g_type_register_static({base_gtype}, \"{class_name}\", &self_info, 0);\
+		self_type = g_type_register_static({base_gtype}, \"{class_name}\", &self_info, 0);\n\
 	}\n\
 	\n\
 	return self_type;\n\
 }\n\
-\n\
+\n\n\
 static void {class_name_low}_class_init({class_name}Class *klass)\n\
 {\n\
 	{gtk_destructor_registration}\n\
 	parent_class = ({base_name}Class*)g_type_class_peek({base_gtype});\n\
 	g_type_class_add_private((gpointer)klass, sizeof({class_name}Private));\n\
 }\n\
-\n\
+\n\n\
 static void {class_name_low}_init({class_name} *self)\n\
 {\n\
 	\n\
@@ -218,27 +223,24 @@ static void cc_dlg_on_base_name_entry_changed(GtkWidget *entry, CreateClassDialo
 static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg);
 
 
-/* I don't want this to be in the plugin API because it can cause leaks if any pointers
- * are NULL -ntrel. */
-/* Frees all passed pointers if they are *ALL* non-NULL.
- * Do not use if any pointers may be NULL.
- * The first argument is nothing special, it will also be freed.
- * The list must be ended with NULL. */
+/* The list must be ended with NULL as an extra check that arg_count is correct. */
 static void
-utils_free_pointers(gpointer first, ...)
+utils_free_pointers(gsize arg_count, ...)
 {
 	va_list a;
-	gpointer sa;
+	gsize i;
+	gpointer ptr;
 
-    for (va_start(a, first);  (sa = va_arg(a, gpointer), sa!=NULL);)
-    {
-    	if (sa != NULL)
-    		g_free(sa);
+	va_start(a, arg_count);
+	for (i = 0; i < arg_count; i++)
+	{
+		ptr = va_arg(a, gpointer);
+		g_free(ptr);
 	}
+	ptr = va_arg(a, gpointer);
+	if (ptr)
+		g_warning("Wrong arg_count!");
 	va_end(a);
-
-    if (first != NULL)
-    	g_free(first);
 }
 
 
@@ -251,30 +253,30 @@ get_template_class_header(ClassInfo *class_info)
 	switch (class_info->type)
 	{
 		case GEANY_CLASS_TYPE_CPP:
-			fileheader = p_templates->get_template_fileheader(GEANY_FILETYPES_CPP, class_info->header);
+			fileheader = templates_get_template_fileheader(GEANY_FILETYPES_CPP, class_info->header);
 			template = g_string_new(templates_cpp_class_header);
-			p_utils->string_replace_all(template, "{fileheader}", fileheader);
-			p_utils->string_replace_all(template, "{header_guard}", class_info->header_guard);
-			p_utils->string_replace_all(template, "{base_include}", class_info->base_include);
-			p_utils->string_replace_all(template, "{class_name}", class_info->class_name);
-			p_utils->string_replace_all(template, "{base_decl}", class_info->base_decl);
-			p_utils->string_replace_all(template, "{constructor_decl}",
+			utils_string_replace_all(template, "{fileheader}", fileheader);
+			utils_string_replace_all(template, "{header_guard}", class_info->header_guard);
+			utils_string_replace_all(template, "{base_include}", class_info->base_include);
+			utils_string_replace_all(template, "{class_name}", class_info->class_name);
+			utils_string_replace_all(template, "{base_decl}", class_info->base_decl);
+			utils_string_replace_all(template, "{constructor_decl}",
 					class_info->constructor_decl);
-			p_utils->string_replace_all(template, "{destructor_decl}",
+			utils_string_replace_all(template, "{destructor_decl}",
 					class_info->destructor_decl);
 			break;
 
 		case GEANY_CLASS_TYPE_GTK:
-			fileheader = p_templates->get_template_fileheader(GEANY_FILETYPES_C, class_info->header);
+			fileheader = templates_get_template_fileheader(GEANY_FILETYPES_C, class_info->header);
 			template = g_string_new(templates_gtk_class_header);
-			p_utils->string_replace_all(template, "{fileheader}", fileheader);
-			p_utils->string_replace_all(template, "{header_guard}", class_info->header_guard);
-			p_utils->string_replace_all(template, "{base_include}", class_info->base_include);
-			p_utils->string_replace_all(template, "{class_name}", class_info->class_name);
-			p_utils->string_replace_all(template, "{class_name_up}", class_info->class_name_up);
-			p_utils->string_replace_all(template, "{class_name_low}", class_info->class_name_low);
-			p_utils->string_replace_all(template, "{base_name}", class_info->base_name);
-			p_utils->string_replace_all(template, "{constructor_decl}",
+			utils_string_replace_all(template, "{fileheader}", fileheader);
+			utils_string_replace_all(template, "{header_guard}", class_info->header_guard);
+			utils_string_replace_all(template, "{base_include}", class_info->base_include);
+			utils_string_replace_all(template, "{class_name}", class_info->class_name);
+			utils_string_replace_all(template, "{class_name_up}", class_info->class_name_up);
+			utils_string_replace_all(template, "{class_name_low}", class_info->class_name_low);
+			utils_string_replace_all(template, "{base_name}", class_info->base_name);
+			utils_string_replace_all(template, "{constructor_decl}",
 					class_info->constructor_decl);
 			break;
 	}
@@ -297,35 +299,35 @@ get_template_class_source(ClassInfo *class_info)
 	switch (class_info->type)
 	{
 		case GEANY_CLASS_TYPE_CPP:
-			fileheader = p_templates->get_template_fileheader(GEANY_FILETYPES_CPP, class_info->source);
+			fileheader = templates_get_template_fileheader(GEANY_FILETYPES_CPP, class_info->source);
 			template = g_string_new(templates_cpp_class_source);
-			p_utils->string_replace_all(template, "{fileheader}", fileheader);
-			p_utils->string_replace_all(template, "{header}", class_info->header);
-			p_utils->string_replace_all(template, "{class_name}", class_info->class_name);
-			p_utils->string_replace_all(template, "{base_include}", class_info->base_include);
-			p_utils->string_replace_all(template, "{base_name}", class_info->base_name);
-			p_utils->string_replace_all(template, "{constructor_impl}",
+			utils_string_replace_all(template, "{fileheader}", fileheader);
+			utils_string_replace_all(template, "{header}", class_info->header);
+			utils_string_replace_all(template, "{class_name}", class_info->class_name);
+			utils_string_replace_all(template, "{base_include}", class_info->base_include);
+			utils_string_replace_all(template, "{base_name}", class_info->base_name);
+			utils_string_replace_all(template, "{constructor_impl}",
 					class_info->constructor_impl);
-			p_utils->string_replace_all(template, "{destructor_impl}",
+			utils_string_replace_all(template, "{destructor_impl}",
 					class_info->destructor_impl);
 			break;
 
 		case GEANY_CLASS_TYPE_GTK:
-			fileheader = p_templates->get_template_fileheader(GEANY_FILETYPES_C, class_info->source);
+			fileheader = templates_get_template_fileheader(GEANY_FILETYPES_C, class_info->source);
 			template = g_string_new(templates_gtk_class_source);
-			p_utils->string_replace_all(template, "{fileheader}", fileheader);
-			p_utils->string_replace_all(template, "{header}", class_info->header);
-			p_utils->string_replace_all(template, "{class_name}", class_info->class_name);
-			p_utils->string_replace_all(template, "{class_name_up}", class_info->class_name_up);
-			p_utils->string_replace_all(template, "{class_name_low}", class_info->class_name_low);
-			p_utils->string_replace_all(template, "{base_name}", class_info->base_name);
-			p_utils->string_replace_all(template, "{base_gtype}", class_info->base_gtype);
-			p_utils->string_replace_all(template, "{destructor_decl}", class_info->destructor_decl);
-			p_utils->string_replace_all(template, "{constructor_impl}",
+			utils_string_replace_all(template, "{fileheader}", fileheader);
+			utils_string_replace_all(template, "{header}", class_info->header);
+			utils_string_replace_all(template, "{class_name}", class_info->class_name);
+			utils_string_replace_all(template, "{class_name_up}", class_info->class_name_up);
+			utils_string_replace_all(template, "{class_name_low}", class_info->class_name_low);
+			utils_string_replace_all(template, "{base_name}", class_info->base_name);
+			utils_string_replace_all(template, "{base_gtype}", class_info->base_gtype);
+			utils_string_replace_all(template, "{destructor_decl}", class_info->destructor_decl);
+			utils_string_replace_all(template, "{constructor_impl}",
 					class_info->constructor_impl);
-			p_utils->string_replace_all(template, "{destructor_impl}",
+			utils_string_replace_all(template, "{destructor_impl}",
 					class_info->destructor_impl);
-			p_utils->string_replace_all(template, "{gtk_destructor_registration}",
+			utils_string_replace_all(template, "{gtk_destructor_registration}",
 					class_info->gtk_destructor_registration);
 			break;
 	}
@@ -353,17 +355,16 @@ void show_dialog_create_class(gint type)
 	cc_dlg->class_type = type;
 
 	cc_dlg->dialog = gtk_dialog_new_with_buttons(_("Create Class"),
-			GTK_WINDOW(app->window),
+			GTK_WINDOW(geany->main_widgets->window),
 			GTK_DIALOG_MODAL,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 			GTK_STOCK_OK, GTK_RESPONSE_OK,
 			NULL);
-	g_signal_connect_swapped(G_OBJECT(cc_dlg->dialog), "destroy",
-			G_CALLBACK(g_free), (gpointer)cc_dlg);
+	g_signal_connect_swapped(cc_dlg->dialog, "destroy", G_CALLBACK(g_free), (gpointer)cc_dlg);
 
-	main_box = p_ui->dialog_vbox_new(GTK_DIALOG(cc_dlg->dialog));
+	main_box = ui_dialog_vbox_new(GTK_DIALOG(cc_dlg->dialog));
 
-	frame = p_ui->frame_new_with_alignment(_("Class"), &align);
+	frame = ui_frame_new_with_alignment(_("Class"), &align);
 	gtk_container_add(GTK_CONTAINER(main_box), frame);
 
 	vbox = gtk_vbox_new(FALSE, 10);
@@ -377,7 +378,7 @@ void show_dialog_create_class(gint type)
 
 	cc_dlg->class_name_entry = gtk_entry_new();
 	gtk_box_pack_start(GTK_BOX(hbox), cc_dlg->class_name_entry, TRUE, TRUE, 0);
-	g_signal_connect(G_OBJECT(cc_dlg->class_name_entry), "changed",
+	g_signal_connect(cc_dlg->class_name_entry, "changed",
 			G_CALLBACK(cc_dlg_on_class_name_entry_changed), cc_dlg);
 
 	hbox = gtk_hbox_new(FALSE, 10);
@@ -398,7 +399,7 @@ void show_dialog_create_class(gint type)
 	cc_dlg->source_entry = gtk_entry_new();
 	gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->source_entry);
 
-	frame = p_ui->frame_new_with_alignment(_("Inheritance"), &align);
+	frame = ui_frame_new_with_alignment(_("Inheritance"), &align);
 	gtk_container_add(GTK_CONTAINER(main_box), frame);
 
 	vbox = gtk_vbox_new(FALSE, 10);
@@ -414,7 +415,7 @@ void show_dialog_create_class(gint type)
 	if (type == GEANY_CLASS_TYPE_GTK)
 		gtk_entry_set_text(GTK_ENTRY(cc_dlg->base_name_entry), "GObject");
 	gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->base_name_entry);
-	g_signal_connect(G_OBJECT(cc_dlg->base_name_entry), "changed",
+	g_signal_connect(cc_dlg->base_name_entry, "changed",
 			G_CALLBACK(cc_dlg_on_base_name_entry_changed), (gpointer)cc_dlg);
 
 	hbox = gtk_hbox_new(FALSE, 10);
@@ -441,11 +442,11 @@ void show_dialog_create_class(gint type)
 		gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
 
 		cc_dlg->base_gtype_entry = gtk_entry_new();
-		gtk_entry_set_text(GTK_ENTRY(cc_dlg->base_gtype_entry), "GTK_TYPE_OBJECT");
+		gtk_entry_set_text(GTK_ENTRY(cc_dlg->base_gtype_entry), "G_TYPE_OBJECT");
 		gtk_container_add(GTK_CONTAINER(hbox), cc_dlg->base_gtype_entry);
 	}
 
-	frame = p_ui->frame_new_with_alignment(_("Options"), &align);
+	frame = ui_frame_new_with_alignment(_("Options"), &align);
 	gtk_container_add(GTK_CONTAINER(main_box), frame);
 
 	vbox = gtk_vbox_new(FALSE, 10);
@@ -465,7 +466,7 @@ void show_dialog_create_class(gint type)
 	{
 		hbox = gtk_hbox_new(FALSE, 10);
 		gtk_container_add(GTK_CONTAINER(vbox), hbox);
-		g_signal_connect(G_OBJECT(cc_dlg->create_constructor_box), "toggled",
+		g_signal_connect(cc_dlg->create_constructor_box, "toggled",
 				G_CALLBACK(cc_dlg_on_set_sensitive_toggled), (gpointer)hbox);
 
 		label = gtk_label_new(_("GTK+ constructor type"));
@@ -555,8 +556,10 @@ static void cc_dlg_on_base_name_entry_changed(GtkWidget *entry, CreateClassDialo
 
 	base_name_splitted = str_case_split(gtk_entry_get_text(GTK_ENTRY(entry)), '_');
 	if (! g_ascii_strncasecmp(gtk_entry_get_text(GTK_ENTRY(entry)), "gtk", 3))
-		tmp = g_strconcat("gtk/", gtk_entry_get_text(GTK_ENTRY(entry)), ".h", NULL);
-	else if (p_utils->str_equal(gtk_entry_get_text(GTK_ENTRY(entry)), "GObject"))
+		/*tmp = g_strconcat("gtk/", gtk_entry_get_text(GTK_ENTRY(entry)), ".h", NULL);*/
+		/* With GTK 2.14 (and later GTK 3), single header includes are encouraged */
+		tmp = g_strdup("gtk/gtk.h");
+	else if (utils_str_equal(gtk_entry_get_text(GTK_ENTRY(entry)), "GObject"))
 		tmp = g_strdup("glib-object.h");
 	else
 		tmp = g_strconcat(gtk_entry_get_text(GTK_ENTRY(entry)), ".h", NULL);
@@ -572,7 +575,7 @@ static void cc_dlg_on_base_name_entry_changed(GtkWidget *entry, CreateClassDialo
 			tmp = g_strdup_printf("%.3s_TYPE%s",
 					base_name_splitted,
 					base_name_splitted + 3);
-		else if (p_utils->str_equal(gtk_entry_get_text(GTK_ENTRY(entry)), "GObject"))
+		else if (utils_str_equal(gtk_entry_get_text(GTK_ENTRY(entry)), "GObject"))
 			tmp = g_strdup("G_TYPE_OBJECT");
 		else
 			tmp = g_strconcat(base_name_splitted, "_TYPE", NULL);
@@ -591,13 +594,13 @@ static void cc_dlg_on_base_name_entry_changed(GtkWidget *entry, CreateClassDialo
 static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 {
 	ClassInfo *class_info;
-	gint idx;
+	GeanyDocument *doc;
 	gchar *text;
 	gchar *tmp;
 
 	g_return_if_fail(cc_dlg != NULL);
 
-	if (p_utils->str_equal(gtk_entry_get_text(GTK_ENTRY(cc_dlg->class_name_entry)), ""))
+	if (utils_str_equal(gtk_entry_get_text(GTK_ENTRY(cc_dlg->class_name_entry)), ""))
 		return;
 
 	class_info = g_new0(ClassInfo, 1);
@@ -606,7 +609,7 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 	tmp = str_case_split(class_info->class_name, '_');
 	class_info->class_name_up = g_ascii_strup(tmp, -1);
 	class_info->class_name_low = g_ascii_strdown(class_info->class_name_up, -1);
-	if (! p_utils->str_equal(gtk_entry_get_text(GTK_ENTRY(cc_dlg->base_name_entry)), ""))
+	if (! utils_str_equal(gtk_entry_get_text(GTK_ENTRY(cc_dlg->base_name_entry)), ""))
 	{
 		class_info->base_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->base_name_entry)));
 		class_info->base_include = g_strdup_printf("\n#include %c%s%c\n",
@@ -629,7 +632,7 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 		case GEANY_CLASS_TYPE_CPP:
 		{
 			class_info->source = g_strdup(gtk_entry_get_text(GTK_ENTRY(cc_dlg->source_entry)));
-			if (! p_utils->str_equal(class_info->base_name, ""))
+			if (! utils_str_equal(class_info->base_name, ""))
 				class_info->base_decl = g_strdup_printf(": public %s", class_info->base_name);
 			else
 				class_info->base_decl = g_strdup("");
@@ -637,7 +640,7 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 			{
 				gchar *base_constructor;
 
-				if (p_utils->str_equal(class_info->base_name, ""))
+				if (utils_str_equal(class_info->base_name, ""))
 					base_constructor = g_strdup("");
 				else
 					base_constructor = g_strdup_printf("\t: %s()\n", class_info->base_name);
@@ -698,10 +701,10 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 						"\tg_object_class->finalize = %s_finalize;\n",
 						class_info->class_name_low);
 				class_info->destructor_decl =
-						g_strdup_printf("static void %s_finalize\t\t\t(GObject *object);\n",
+						g_strdup_printf("static void %s_finalize  \t\t\t(GObject *object);\n",
 						class_info->class_name_low);
 				class_info->destructor_impl = g_strdup_printf("\n"
-						"void %s_finalize(GObject *object)\n"
+						"static void %s_finalize(GObject *object)\n"
 						"{\n"
 						"\t%s *self;\n\n"
 						"\tg_return_if_fail(object != NULL);\n"
@@ -726,23 +729,23 @@ static void cc_dlg_on_create_class(CreateClassDialog *cc_dlg)
 	}
 
 	/* only create the files if the filename is not empty */
-	if (! p_utils->str_equal(class_info->source, ""))
+	if (! utils_str_equal(class_info->source, ""))
 	{
 		text = get_template_class_source(class_info);
-		idx = p_document->new_file(class_info->source, NULL, NULL);
-		p_sci->set_text(doc_list[idx].sci, text);
+		doc = document_new_file(class_info->source, NULL, NULL);
+		sci_set_text(doc->editor->sci, text);
 		g_free(text);
 	}
 
-	if (! p_utils->str_equal(class_info->header, ""))
+	if (! utils_str_equal(class_info->header, ""))
 	{
 		text = get_template_class_header(class_info);
-		idx = p_document->new_file(class_info->header, NULL, NULL);
-		p_sci->set_text(doc_list[idx].sci, text);
+		doc = document_new_file(class_info->header, NULL, NULL);
+		sci_set_text(doc->editor->sci, text);
 		g_free(text);
 	}
 
-	utils_free_pointers(tmp, class_info->class_name, class_info->class_name_up,
+	utils_free_pointers(17, tmp, class_info->class_name, class_info->class_name_up,
 		class_info->base_name, class_info->class_name_low, class_info->base_include,
 		class_info->header, class_info->header_guard, class_info->source, class_info->base_decl,
 		class_info->constructor_decl, class_info->constructor_impl,
@@ -767,19 +770,15 @@ on_menu_create_gtk_class_activate      (GtkMenuItem     *menuitem,
 }
 
 
-void init(GeanyData *data)
+void plugin_init(GeanyData *data)
 {
 	GtkWidget *menu_create_class1;
-	GtkWidget *image1861;
 	GtkWidget *menu_create_class1_menu;
 	GtkWidget *menu_create_cpp_class;
 	GtkWidget *menu_create_gtk_class;
 
-	menu_create_class1 = gtk_image_menu_item_new_with_mnemonic (_("Create Cla_ss"));
-	gtk_container_add (GTK_CONTAINER (data->tools_menu), menu_create_class1);
-
-	image1861 = gtk_image_new_from_stock ("gtk-add", GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_create_class1), image1861);
+	menu_create_class1 = ui_image_menu_item_new (GTK_STOCK_ADD, _("Create Cla_ss"));
+	gtk_container_add (GTK_CONTAINER (geany->main_widgets->tools_menu), menu_create_class1);
 
 	menu_create_class1_menu = gtk_menu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_create_class1), menu_create_class1_menu);
@@ -790,20 +789,21 @@ void init(GeanyData *data)
 	menu_create_gtk_class = gtk_menu_item_new_with_mnemonic (_("_GTK+ Class"));
 	gtk_container_add (GTK_CONTAINER (menu_create_class1_menu), menu_create_gtk_class);
 
-	g_signal_connect ((gpointer) menu_create_cpp_class, "activate",
+	g_signal_connect(menu_create_cpp_class, "activate",
 		G_CALLBACK (on_menu_create_cpp_class_activate),
 		NULL);
-	g_signal_connect ((gpointer) menu_create_gtk_class, "activate",
+	g_signal_connect(menu_create_gtk_class, "activate",
 		G_CALLBACK (on_menu_create_gtk_class_activate),
 		NULL);
 
 	gtk_widget_show_all(menu_create_class1);
 
-	plugin_fields->menu_item = menu_create_class1;
+	ui_add_document_sensitive(menu_create_class1);
+	main_menu_item = menu_create_class1;
 }
 
 
-void cleanup(void)
+void plugin_cleanup(void)
 {
-	gtk_widget_destroy(plugin_fields->menu_item);
+	gtk_widget_destroy(main_menu_item);
 }
