@@ -1,8 +1,8 @@
 /*
  *      demoplugin.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: demoplugin.c 3085 2008-10-14 17:03:44Z ntrel $
+ * $Id: demoplugin.c 3522 2009-01-28 17:55:58Z eht16 $
  */
 
 /**
@@ -30,17 +30,22 @@
  * cd plugins
  * make demoplugin.so
  *
- * Then copy or symlink the plugins/demoplugin.so file to ~/.geany/plugins
+ * Then copy or symlink the plugins/demoplugin.so file to ~/.config/geany/plugins
  * - it will be loaded at next startup.
  */
 
 
 #include "geany.h"		/* for the GeanyApp data type */
 #include "support.h"	/* for the _() translation macro (see also po/POTFILES.in) */
+#include "editor.h"		/* for the declaration of the GeanyEditor struct, not strictly necessary
+						   as it will be also included by plugindata.h */
+#include "document.h"	/* for the declaration of the GeanyDocument struct */
 #include "ui_utils.h"
+#include "Scintilla.h"	/* for the SCNotification struct */
 
 #include "plugindata.h"		/* this defines the plugin API */
-#include "pluginmacros.h"	/* some useful macros to save typing */
+#include "geanyfunctions.h"	/* this wraps geany_functions function pointers */
+
 
 
 /* These items are set by Geany before plugin_init() is called. */
@@ -51,7 +56,7 @@ GeanyFunctions	*geany_functions;
 
 /* Check that the running Geany supports the plugin API used below, and check
  * for binary compatibility. */
-PLUGIN_VERSION_CHECK(100)
+PLUGIN_VERSION_CHECK(112)
 
 /* All plugins must set name, description, version and author. */
 PLUGIN_SET_INFO(_("Demo"), _("Example plugin."), VERSION, _("The Geany developer team"))
@@ -62,6 +67,61 @@ static GtkWidget *main_menu_item = NULL;
 static gchar *welcome_text = NULL;
 
 
+
+static gboolean on_editor_notify(GObject *object, GeanyEditor *editor,
+								 SCNotification *nt, gpointer data)
+{
+	/* For detailed documentation about the SCNotification struct, please see
+	 * http://www.scintilla.org/ScintillaDoc.html#Notifications. */
+	switch (nt->nmhdr.code)
+	{
+		case SCN_UPDATEUI:
+			/* This notification is sent very often, you should not do time-consuming tasks here */
+			break;
+		case SCN_CHARADDED:
+			/* For demonstrating purposes simply print the typed character in the status bar */
+			ui_set_statusbar(FALSE, _("Typed character: %c"), nt->ch);
+			break;
+		case SCN_URIDROPPED:
+		{
+			/* Show a message dialog with the dropped URI list when files (i.e. a list of
+			 * filenames) is dropped to the editor widget) */
+			if (nt->text != NULL)
+			{
+				GtkWidget *dialog;
+
+				dialog = gtk_message_dialog_new(
+					GTK_WINDOW(geany->main_widgets->window),
+					GTK_DIALOG_DESTROY_WITH_PARENT,
+					GTK_MESSAGE_INFO,
+					GTK_BUTTONS_OK,
+					_("The following files were dropped:"));
+				gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog),
+					"%s", nt->text);
+
+				gtk_dialog_run(GTK_DIALOG(dialog));
+				gtk_widget_destroy(dialog);
+			}
+			/* we return TRUE here which prevents Geany from processing the SCN_URIDROPPED
+			 * notification, i.e. Geany won't open the passed files */
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
+
+PluginCallback plugin_callbacks[] =
+{
+	/* Set 'after' (third field) to TRUE to run the callback @a after the default handler.
+	 * If 'after' is FALSE, the callback is run @a before the default handler, so the plugin
+	 * can prevent Geany from processing the notification. Use this with care. */
+	{ "editor-notify", (GCallback) &on_editor_notify, FALSE, NULL },
+	{ NULL, NULL, FALSE, NULL }
+};
+
+
 /* Callback when the menu item is clicked. */
 static void
 item_activate(GtkMenuItem *menuitem, gpointer gdata)
@@ -95,7 +155,7 @@ void plugin_init(GeanyData *data)
 	g_signal_connect(demo_item, "activate", G_CALLBACK(item_activate), NULL);
 
 	/* make the menu item sensitive only when documents are open */
-	p_ui->add_document_sensitive(demo_item);
+	ui_add_document_sensitive(demo_item);
 	/* keep a pointer to the menu item, so we can remove it when the plugin is unloaded */
 	main_menu_item = demo_item;
 
@@ -119,7 +179,7 @@ on_configure_response(GtkDialog *dialog, gint response, gpointer user_data)
 		 * (e.g. using GLib's GKeyFile API)
 		 * all plugin specific files should be created in:
 		 * geany->app->configdir G_DIR_SEPARATOR_S plugins G_DIR_SEPARATOR_S pluginname G_DIR_SEPARATOR_S
-		 * e.g. this could be: ~/.geany/plugins/Demo/, please use geany->app->configdir */
+		 * e.g. this could be: ~/.config/geany/plugins/Demo/, please use geany->app->configdir */
 	}
 }
 
