@@ -1,8 +1,8 @@
 /*
  *      htmlchars.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: htmlchars.c 3085 2008-10-14 17:03:44Z ntrel $
+ * $Id: htmlchars.c 3571 2009-02-10 21:11:25Z eht16 $
  */
 
 /* HTML Characters plugin (Inserts HTML character entities like '&amp;') */
@@ -32,7 +32,7 @@
 #include "keybindings.h"
 #include "ui_utils.h"
 #include "utils.h"
-#include "pluginmacros.h"
+#include "geanyfunctions.h"
 
 
 GeanyData		*geany_data;
@@ -89,7 +89,7 @@ static void tools_show_dialog_insert_special_chars(void)
 					_("Special Characters"), GTK_WINDOW(geany->main_widgets->window),
 					GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 					_("_Insert"), GTK_RESPONSE_OK, NULL);
-		vbox = p_ui->dialog_vbox_new(GTK_DIALOG(sc_dialog));
+		vbox = ui_dialog_vbox_new(GTK_DIALOG(sc_dialog));
 		gtk_box_set_spacing(GTK_BOX(vbox), 6);
 		gtk_widget_set_name(sc_dialog, "GeanyDialog");
 
@@ -151,7 +151,7 @@ static void sc_fill_store(GtkTreeStore *store)
 {
 	GtkTreeIter iter;
 	GtkTreeIter *parent_iter = NULL;
-	guint i;
+	guint i, len;
 
 	gchar *chars[][2] =
 		{
@@ -416,7 +416,8 @@ static void sc_fill_store(GtkTreeStore *store)
 			{ "ƒ", "&fnof;" },
 		};
 
-	for (i = 0; i < G_N_ELEMENTS(chars); i++)
+	len = G_N_ELEMENTS(chars);
+	for (i = 0; i < len; i++)
 	{
 		if (chars[i][1] == NULL)
 		{	/* add a category */
@@ -439,18 +440,18 @@ static void sc_fill_store(GtkTreeStore *store)
  * returns only TRUE if a valid selection(i.e. no category) could be found */
 static gboolean sc_insert(GtkTreeModel *model, GtkTreeIter *iter)
 {
-	GeanyDocument *doc = p_document->get_current();
+	GeanyDocument *doc = document_get_current();
 	gboolean result = FALSE;
 
 	if (doc != NULL)
 	{
 		gchar *str;
-		gint pos = p_sci->get_current_position(doc->editor->sci);
+		gint pos = sci_get_current_position(doc->editor->sci);
 
 		gtk_tree_model_get(model, iter, COLUMN_HTML_NAME, &str, -1);
 		if (NZV(str))
 		{
-			p_sci->insert_text(doc->editor->sci, pos, str);
+			sci_insert_text(doc->editor->sci, pos, str);
 			g_free(str);
 			result = TRUE;
 		}
@@ -532,11 +533,11 @@ void plugin_init(GeanyData *data)
 	g_signal_connect(menu_item, "activate", G_CALLBACK(item_activate), NULL);
 
 	/* disable menu_item when there are no documents open */
-	p_ui->add_document_sensitive(menu_item);
+	ui_add_document_sensitive(menu_item);
 	main_menu_item = menu_item;
 
 	/* setup keybindings */
-	p_keybindings->set_item(plugin_key_group, KB_INSERT_HTML_CHARS, kb_activate,
+	keybindings_set_item(plugin_key_group, KB_INSERT_HTML_CHARS, kb_activate,
 		0, 0, "insert_html_chars", kb_label, menu_item);
 }
 
