@@ -1,8 +1,8 @@
 /*
  *      splitwindow.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: splitwindow.c 3112 2008-10-18 10:54:49Z eht16 $
+ * $Id: splitwindow.c 3460 2009-01-11 18:29:39Z eht16 $
  */
 
 /* Split Window plugin. */
@@ -34,7 +34,7 @@
 #include "document.h"
 #include "editor.h"
 #include "plugindata.h"
-#include "pluginmacros.h"
+#include "geanyfunctions.h"
 
 
 PLUGIN_VERSION_CHECK(GEANY_API_VERSION)
@@ -49,7 +49,7 @@ GeanyFunctions	*geany_functions;
 enum State
 {
 	STATE_SPLIT_HORIZONTAL,
-	/* STATE_SPLIT_VERTICAL, */
+	STATE_SPLIT_VERTICAL,
 	STATE_UNSPLIT,
 	STATE_COUNT
 };
@@ -58,6 +58,7 @@ static struct
 {
 	GtkWidget *main;
 	GtkWidget *horizontal;
+	GtkWidget *vertical;
 	GtkWidget *unsplit;
 }
 menu_items;
@@ -82,7 +83,7 @@ static void on_unsplit(GtkMenuItem *menuitem, gpointer user_data);
 
 static gint sci_get_value(ScintillaObject *sci, gint message_id, gint param)
 {
-	return p_sci->send_message(sci, message_id, param, 0);
+	return scintilla_send_message(sci, message_id, param, 0);
 }
 
 
@@ -95,13 +96,13 @@ static void set_styles(ScintillaObject *oldsci, ScintillaObject *newsci)
 		gint val;
 
 		val = sci_get_value(oldsci, SCI_STYLEGETFORE, style_id);
-		p_sci->send_message(newsci, SCI_STYLESETFORE, style_id, val);
+		scintilla_send_message(newsci, SCI_STYLESETFORE, style_id, val);
 		val = sci_get_value(oldsci, SCI_STYLEGETBACK, style_id);
-		p_sci->send_message(newsci, SCI_STYLESETBACK, style_id, val);
+		scintilla_send_message(newsci, SCI_STYLESETBACK, style_id, val);
 		val = sci_get_value(oldsci, SCI_STYLEGETBOLD, style_id);
-		p_sci->send_message(newsci, SCI_STYLESETBOLD, style_id, val);
+		scintilla_send_message(newsci, SCI_STYLESETBOLD, style_id, val);
 		val = sci_get_value(oldsci, SCI_STYLEGETITALIC, style_id);
-		p_sci->send_message(newsci, SCI_STYLESETITALIC, style_id, val);
+		scintilla_send_message(newsci, SCI_STYLESETITALIC, style_id, val);
 	}
 }
 
@@ -109,8 +110,8 @@ static void set_styles(ScintillaObject *oldsci, ScintillaObject *newsci)
 static void sci_set_font(ScintillaObject *sci, gint style, const gchar *font,
 	gint size)
 {
-	p_sci->send_message(sci, SCI_STYLESETFONT, style, (sptr_t) font);
-	p_sci->send_message(sci, SCI_STYLESETSIZE, style, size);
+	scintilla_send_message(sci, SCI_STYLESETFONT, style, (sptr_t) font);
+	scintilla_send_message(sci, SCI_STYLESETSIZE, style, size);
 }
 
 
@@ -120,7 +121,7 @@ static void update_font(ScintillaObject *current, ScintillaObject *sci)
 	gint size;
 	gchar font_name[1024]; /* should be big enough */
 
-	p_sci->send_message(current, SCI_STYLEGETFONT, 0, (sptr_t)font_name);
+	scintilla_send_message(current, SCI_STYLEGETFONT, 0, (sptr_t)font_name);
 	size = sci_get_value(current, SCI_STYLEGETSIZE, 0);
 
 	for (style_id = 0; style_id <= 127; style_id++)
@@ -141,16 +142,16 @@ static void set_line_numbers(ScintillaObject * sci, gboolean set, gint extra_wid
 	if (set)
 	{
 		gchar tmp_str[15];
-		gint len = p_sci->send_message(sci, SCI_GETLINECOUNT, 0, 0);
+		gint len = scintilla_send_message(sci, SCI_GETLINECOUNT, 0, 0);
 		gint width;
 		g_snprintf(tmp_str, 15, "_%d%d", len, extra_width);
-		width = p_sci->send_message(sci, SCI_TEXTWIDTH, STYLE_LINENUMBER, (sptr_t) tmp_str);
-		p_sci->send_message(sci, SCI_SETMARGINWIDTHN, 0, width);
-		p_sci->send_message(sci, SCI_SETMARGINSENSITIVEN, 0, FALSE); /* use default behaviour */
+		width = scintilla_send_message(sci, SCI_TEXTWIDTH, STYLE_LINENUMBER, (sptr_t) tmp_str);
+		scintilla_send_message(sci, SCI_SETMARGINWIDTHN, 0, width);
+		scintilla_send_message(sci, SCI_SETMARGINSENSITIVEN, 0, FALSE); /* use default behaviour */
 	}
 	else
 	{
-		p_sci->send_message(sci, SCI_SETMARGINWIDTHN, 0, 0 );
+		scintilla_send_message(sci, SCI_SETMARGINWIDTHN, 0, 0 );
 	}
 }
 
@@ -162,20 +163,20 @@ static void sync_to_current(ScintillaObject *sci, ScintillaObject *current)
 	gint pos;
 
 	/* set the new sci widget to view the existing Scintilla document */
-	sdoc = (gpointer) p_sci->send_message(current, SCI_GETDOCPOINTER, 0, 0);
-	p_sci->send_message(sci, SCI_SETDOCPOINTER, 0, GPOINTER_TO_INT(sdoc));
+	sdoc = (gpointer) scintilla_send_message(current, SCI_GETDOCPOINTER, 0, 0);
+	scintilla_send_message(sci, SCI_SETDOCPOINTER, 0, GPOINTER_TO_INT(sdoc));
 
 	update_font(current, sci);
-	lexer = p_sci->send_message(current, SCI_GETLEXER, 0, 0);
-	p_sci->send_message(sci, SCI_SETLEXER, lexer, 0);
+	lexer = scintilla_send_message(current, SCI_GETLEXER, 0, 0);
+	scintilla_send_message(sci, SCI_SETLEXER, lexer, 0);
 	set_styles(current, sci);
 
-	pos = p_sci->get_current_position(current);
-	p_sci->set_current_position(sci, pos, TRUE);
+	pos = sci_get_current_position(current);
+	sci_set_current_position(sci, pos, TRUE);
 
 	/* override some defaults */
 	set_line_numbers(sci, TRUE, 0);
-	p_sci->send_message(sci, SCI_SETMARGINWIDTHN, 1, 0 ); /* hide marker margin */
+	scintilla_send_message(sci, SCI_SETMARGINWIDTHN, 1, 0 ); /* hide marker margin */
 }
 
 
@@ -188,7 +189,7 @@ static void set_editor(EditWindow *editwin, GeanyEditor *editor)
 	if (editwin->sci != NULL)
 		gtk_widget_destroy(GTK_WIDGET(editwin->sci));
 
-	editwin->sci = p_editor->create_widget(editor);
+	editwin->sci = editor_create_widget(editor);
 	gtk_widget_show(GTK_WIDGET(editwin->sci));
 	gtk_container_add(GTK_CONTAINER(editwin->vbox), GTK_WIDGET(editwin->sci));
 
@@ -201,7 +202,9 @@ static void set_editor(EditWindow *editwin, GeanyEditor *editor)
 static void set_state(enum State id)
 {
 	gtk_widget_set_sensitive(menu_items.horizontal,
-		id != STATE_SPLIT_HORIZONTAL);
+		(id != STATE_SPLIT_HORIZONTAL) && (id != STATE_SPLIT_VERTICAL));
+	gtk_widget_set_sensitive(menu_items.vertical,
+		(id != STATE_SPLIT_HORIZONTAL) && (id != STATE_SPLIT_VERTICAL));
 	gtk_widget_set_sensitive(menu_items.unsplit,
 		id != STATE_UNSPLIT);
 
@@ -212,12 +215,10 @@ static void set_state(enum State id)
 static GtkWidget *create_tool_button(const gchar *label, const gchar *stock_id)
 {
 	GtkToolItem *item;
-	GtkTooltips *tooltips = GTK_TOOLTIPS(p_support->lookup_widget(
-		geany->main_widgets->window, "tooltips"));
 
 	item = gtk_tool_button_new(NULL, label);
 	gtk_tool_button_set_icon_name(GTK_TOOL_BUTTON(item), stock_id);
-	gtk_tool_item_set_tooltip(item, tooltips, label, NULL);
+	ui_widget_set_tooltip_text(GTK_WIDGET(item), label);
 
 	return GTK_WIDGET(item);
 }
@@ -225,7 +226,7 @@ static GtkWidget *create_tool_button(const gchar *label, const gchar *stock_id)
 
 static void on_refresh(void)
 {
-	GeanyDocument *doc = p_document->get_current();
+	GeanyDocument *doc = document_get_current();
 
 	g_return_if_fail(doc);
 	g_return_if_fail(edit_window.sci);
@@ -276,25 +277,26 @@ static GtkWidget *create_toolbar(void)
 }
 
 
-static void on_split_view(GtkMenuItem *menuitem, gpointer user_data)
+static void split_view(gboolean horizontal)
 {
 	GtkWidget *notebook = geany_data->main_widgets->notebook;
 	GtkWidget *parent = gtk_widget_get_parent(notebook);
 	GtkWidget *pane, *toolbar, *box;
-	GeanyDocument *doc = p_document->get_current();
+	GeanyDocument *doc = document_get_current();
 	gint width = notebook->allocation.width / 2;
-
-	set_state(STATE_SPLIT_HORIZONTAL);
+	gint height = notebook->allocation.height / 2;
 
 	g_return_if_fail(doc);
 	g_return_if_fail(edit_window.editor == NULL);
 
+	set_state(horizontal ? STATE_SPLIT_HORIZONTAL : STATE_SPLIT_VERTICAL);
+
 	/* temporarily put document notebook in main vbox (scintilla widgets must stay
 	 * in a visible parent window, otherwise there are X selection and scrollbar issues) */
 	gtk_widget_reparent(notebook,
-		p_support->lookup_widget(geany->main_widgets->window, "vbox1"));
+		ui_lookup_widget(geany->main_widgets->window, "vbox1"));
 
-	pane = gtk_hpaned_new();
+	pane = horizontal ? gtk_hpaned_new() : gtk_vpaned_new();
 	gtk_container_add(GTK_CONTAINER(parent), pane);
 	gtk_widget_reparent(notebook, pane);
 
@@ -306,11 +308,30 @@ static void on_split_view(GtkMenuItem *menuitem, gpointer user_data)
 
 	set_editor(&edit_window, doc->editor);
 
-	gtk_paned_set_position(GTK_PANED(pane), width);
+	if (horizontal)
+	{
+		gtk_paned_set_position(GTK_PANED(pane), width);
+	}
+	else
+	{
+		gtk_paned_set_position(GTK_PANED(pane), height);
+	}
 	gtk_widget_show_all(pane);
 }
 
 
+static void on_split_horizontally(GtkMenuItem *menuitem, gpointer user_data)
+{
+	split_view(TRUE);
+}
+
+
+static void on_split_vertically(GtkMenuItem *menuitem, gpointer user_data)
+{
+	split_view(FALSE);
+}
+
+
 static void on_unsplit(GtkMenuItem *menuitem, gpointer user_data)
 {
 	GtkWidget *notebook = geany_data->main_widgets->notebook;
@@ -324,7 +345,7 @@ static void on_unsplit(GtkMenuItem *menuitem, gpointer user_data)
 	/* temporarily put document notebook in main vbox (scintilla widgets must stay
 	 * in a visible parent window, otherwise there are X selection and scrollbar issues) */
 	gtk_widget_reparent(notebook,
-		p_support->lookup_widget(geany->main_widgets->window, "vbox1"));
+		ui_lookup_widget(geany->main_widgets->window, "vbox1"));
 
 	gtk_widget_destroy(pane);
 	edit_window.editor = NULL;
@@ -339,14 +360,19 @@ void plugin_init(GeanyData *data)
 
 	menu_items.main = item = gtk_menu_item_new_with_mnemonic(_("_Split Window"));
 	gtk_menu_append(geany_data->main_widgets->tools_menu, item);
-	p_ui->add_document_sensitive(item);
+	ui_add_document_sensitive(item);
 
 	menu = gtk_menu_new();
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_items.main), menu);
 
 	menu_items.horizontal = item =
 		gtk_menu_item_new_with_mnemonic(_("_Horizontally"));
-	g_signal_connect(item, "activate", G_CALLBACK(on_split_view), NULL);
+	g_signal_connect(item, "activate", G_CALLBACK(on_split_horizontally), NULL);
+	gtk_menu_append(menu, item);
+
+	menu_items.vertical = item =
+		gtk_menu_item_new_with_mnemonic(_("_Vertically"));
+	g_signal_connect(item, "activate", G_CALLBACK(on_split_vertically), NULL);
 	gtk_menu_append(menu, item);
 
 	menu_items.unsplit = item =
@@ -384,7 +410,7 @@ PluginCallback plugin_callbacks[] =
 };
 
 
-void plugin_cleanup()
+void plugin_cleanup(void)
 {
 	if (plugin_state != STATE_UNSPLIT)
 		on_unsplit(NULL, NULL);
