@@ -1,8 +1,8 @@
 /*
  *      build.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: build.c 3026 2008-09-30 20:30:38Z eht16 $
+ * $Id: build.c 3570 2009-02-10 21:10:50Z eht16 $
  */
 
 /*
@@ -56,6 +56,8 @@
 #include "project.h"
 #include "editor.h"
 #include "win32.h"
+#include "toolbar.h"
+#include "geanymenubuttonaction.h"
 
 
 GeanyBuildInfo build_info = {GBO_COMPILE, 0, NULL, GEANY_FILETYPES_NONE, NULL};
@@ -82,6 +84,8 @@ enum
 	LATEX_CMD_VIEW_PDF
 };
 
+static GeanyBuildType last_toolbutton_action = GBO_BUILD;
+
 static BuildMenuItems default_menu_items =
 	{NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
 static BuildMenuItems latex_menu_items =
@@ -92,6 +96,14 @@ static struct
 {
 	GtkWidget	*run_button;
 	GtkWidget	*compile_button;
+	GtkWidget	*build_button;
+	GtkAction	*build_action;
+
+	GtkWidget	*toolitem_build;
+	GtkWidget	*toolitem_make_all;
+	GtkWidget	*toolitem_make_custom;
+	GtkWidget	*toolitem_make_object;
+	GtkWidget	*toolitem_set_args;
 }
 widgets;
 
@@ -178,7 +190,7 @@ static GPid build_view_tex_file(GeanyDocument *doc, gint mode)
 	if (g_stat(locale_filename, &st) != 0)
 	{
 		ui_set_statusbar(TRUE, _("Failed to view %s (make sure it is already compiled)"), view_file);
-		utils_free_pointers(executable, view_file, locale_filename, NULL);
+		utils_free_pointers(3, executable, view_file, locale_filename, NULL);
 
 		return (GPid) 1;
 	}
@@ -213,7 +225,7 @@ static GPid build_view_tex_file(GeanyDocument *doc, gint mode)
 			_("Could not find terminal \"%s\" "
 				"(check path for Terminal tool setting in Preferences)"), tool_prefs.term_cmd);
 
-		utils_free_pointers(executable, view_file, locale_filename, cmd_string, locale_cmd_string,
+		utils_free_pointers(6, executable, view_file, locale_filename, cmd_string, locale_cmd_string,
 										locale_term_cmd, NULL);
 		g_strfreev(term_argv);
 		return (GPid) 1;
@@ -227,12 +239,12 @@ static GPid build_view_tex_file(GeanyDocument *doc, gint mode)
 	{
 		ui_set_statusbar(TRUE, _("Failed to execute \"%s\" (start-script could not be created)"),
 													executable);
-		utils_free_pointers(executable, view_file, locale_filename, cmd_string, locale_cmd_string,
+		utils_free_pointers(7, executable, view_file, locale_filename, cmd_string, locale_cmd_string,
 										locale_term_cmd, working_dir, NULL);
 		g_strfreev(term_argv);
 		return (GPid) 1;
 	}
-	g_free(working_dir);
+	g_free(script_name);
 
 	argv = g_new0(gchar *, term_argv_len + 3);
 	for (i = 0; i < term_argv_len; i++)
@@ -244,28 +256,27 @@ static GPid build_view_tex_file(GeanyDocument *doc, gint mode)
 		if (strstr(argv[0], "cmd.exe") != NULL)
 		{
 			argv[term_argv_len   ]  = g_strdup("/Q /C");
-			argv[term_argv_len + 1] = script_name;
+			argv[term_argv_len + 1] = g_strconcat("/bin/sh ", RUN_SCRIPT_CMD, NULL);
 		}
 		else
 		{
-			argv[term_argv_len    ] = script_name;
+			argv[term_argv_len    ] = g_strconcat("/bin/sh ", RUN_SCRIPT_CMD, NULL);
 			argv[term_argv_len + 1] = NULL;
 		}
 #else
 	argv[term_argv_len   ]  = g_strdup("-e");
-	argv[term_argv_len + 1] = script_name;
+	argv[term_argv_len + 1] = g_strconcat("/bin/sh ", RUN_SCRIPT_CMD, NULL);
 #endif
 	argv[term_argv_len + 2] = NULL;
 
-
-	if (! g_spawn_async(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
+	if (! g_spawn_async(working_dir, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD,
 						NULL, NULL, &(run_info.pid), &error))
 	{
 		geany_debug("g_spawn_async() failed: %s", error->message);
 		ui_set_statusbar(TRUE, _("Process failed (%s)"), error->message);
 
-		utils_free_pointers(executable, view_file, locale_filename, cmd_string, locale_cmd_string,
-										locale_term_cmd, NULL);
+		utils_free_pointers(7, executable, view_file, locale_filename, cmd_string,
+			locale_cmd_string, locale_term_cmd, working_dir, NULL);
 		g_strfreev(argv);
 		g_strfreev(term_argv);
 		g_error_free(error);
@@ -280,8 +291,8 @@ static GPid build_view_tex_file(GeanyDocument *doc, gint mode)
 		build_menu_update(doc);
 	}
 
-	utils_free_pointers(executable, view_file, locale_filename, cmd_string, locale_cmd_string,
-										locale_term_cmd, NULL);
+	utils_free_pointers(7, executable, view_file, locale_filename, cmd_string,
+		locale_cmd_string, locale_term_cmd, working_dir, NULL);
 	g_strfreev(argv);
 	g_strfreev(term_argv);
 
@@ -381,7 +392,7 @@ static void clear_errors(GeanyDocument *doc)
 		case GBO_COMPILE:
 		case GBO_MAKE_OBJECT:
 			g_return_if_fail(doc);
-			editor_clear_indicators(doc->editor);
+			editor_indicator_clear_errors(doc->editor);
 			break;
 
 		case GBO_BUILD:
@@ -393,7 +404,7 @@ static void clear_errors(GeanyDocument *doc)
 			for (i = 0; i < documents_array->len; i++)
 			{
 				if (documents[i]->is_valid)
-					editor_clear_indicators(documents[i]->editor);
+					editor_indicator_clear_errors(documents[i]->editor);
 			}
 			break;
 		}
@@ -481,7 +492,7 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 
 	gtk_list_store_clear(msgwindow.store_compiler);
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_COMPILER);
-	msgwin_compiler_add_fmt(COLOR_BLUE, _("%s (in directory: %s)"), utf8_cmd_string, utf8_working_dir);
+	msgwin_compiler_add(COLOR_BLUE, _("%s (in directory: %s)"), utf8_cmd_string, utf8_working_dir);
 	g_free(utf8_working_dir);
 	g_free(utf8_cmd_string);
 
@@ -507,6 +518,7 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 	{
 		g_child_watch_add(build_info.pid, (GChildWatchFunc) build_exit_cb, NULL);
 		build_menu_update(doc);
+		ui_progress_bar_start(NULL);
 	}
 
 	/* use GIOChannels to monitor stdout and stderr */
@@ -523,70 +535,6 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 }
 
 
-/* Checks if the executable file corresponding to document idx exists.
- * Returns the name part of the filename, without extension.
- * Returns NULL if executable file doesn't exist. */
-static gchar *get_build_executable(const gchar *locale_filename, gboolean check_exists,
-		filetype_id ft_id)
-{
-	gchar *long_executable = NULL;
-	struct stat st;
-
-	long_executable = utils_remove_ext_from_filename(locale_filename);
-#ifdef G_OS_WIN32
-	setptr(long_executable, g_strconcat(long_executable, ".exe", NULL));
-#endif
-
-	if (check_exists)
-	{
-		gchar *check_executable = NULL;
-
-		/* add .class extension for JAVA source files (only for stat) */
-		if (ft_id == GEANY_FILETYPES_JAVA)
-		{
-#ifdef G_OS_WIN32
-			gchar *tmp;
-			/* there is already the extension .exe, so first remove it and then add .class */
-			tmp = utils_remove_ext_from_filename(long_executable);
-			check_executable = g_strconcat(tmp, ".class", NULL);
-			/* store the filename without "exe" extension for Java, tmp will be freed by setptr */
-			setptr(long_executable, tmp);
-#else
-			check_executable = g_strconcat(long_executable, ".class", NULL);
-#endif
-		}
-		else
-			check_executable = g_strdup(long_executable);
-
-		/* check for filename extension and abort if filename doesn't have one */
-		if (utils_str_equal(locale_filename, check_executable))
-		{
-			ui_set_statusbar(TRUE, _("Command stopped because the current file has no extension."));
-			utils_beep();
-			g_free(check_executable);
-			return NULL;
-		}
-
-		/* check whether executable exists */
-		if (g_stat(check_executable, &st) != 0)
-		{
-			gchar *utf8_check_executable = utils_get_utf8_from_locale(check_executable);
-
-			ui_set_statusbar(TRUE, _("Failed to execute \"%s\" (make sure it is already built)"),
-														utf8_check_executable);
-			g_free(utf8_check_executable);
-			g_free(check_executable);
-			return NULL;
-		}
-		g_free(check_executable);
-	}
-
-	/* remove path */
-	setptr(long_executable, g_path_get_basename(long_executable));
-	return long_executable;
-}
-
-
 /* Returns: NULL if there was an error, or the working directory the script was created in.
  * vte_cmd_nonscript is the location of a string which is filled with the command to be used
  * when vc->skip_run_script is set, otherwise it will be set to NULL */
@@ -596,7 +544,6 @@ static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript)
 	gboolean have_project;
 	GeanyProject *project = app->project;
 	GeanyFiletype *ft = doc->file_type;
-	gboolean check_exists;
 	gchar	*cmd = NULL;
 	gchar	*executable = NULL;
 	gchar	*working_dir = NULL;
@@ -614,13 +561,10 @@ static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript)
 		project->run_cmd :
 		ft->programs->run_cmd;
 
-	/* only check for existing executable, if executable is required by %e */
-	check_exists = (strstr(cmd, "%e") != NULL);
-	executable = get_build_executable(locale_filename, check_exists, FILETYPE_ID(ft));
-	if (executable == NULL)
+	if (strstr(cmd, "%e") != NULL)
 	{
-		g_free(locale_filename);
-		return NULL;
+		executable = utils_remove_ext_from_filename(locale_filename);
+		setptr(executable, g_path_get_basename(executable));
 	}
 
 	if (have_project)
@@ -641,7 +585,7 @@ static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript)
 		gchar *utf8_working_dir = utils_get_utf8_from_locale(working_dir);
 
 		ui_set_statusbar(TRUE, _("Failed to change the working directory to \"%s\""), utf8_working_dir);
-		utils_free_pointers(utf8_working_dir, working_dir, executable, locale_filename, NULL);
+		utils_free_pointers(4, utf8_working_dir, working_dir, executable, locale_filename, NULL);
 		return NULL;
 	}
 
@@ -660,7 +604,7 @@ static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript)
 			if (vte_cmd_nonscript != NULL)
 				*vte_cmd_nonscript = cmd;
 
-			utils_free_pointers(executable, locale_filename, NULL);
+			utils_free_pointers(2, executable, locale_filename, NULL);
 			return working_dir;
 		}
 		else
@@ -682,7 +626,7 @@ static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript)
 		g_free(utf8_cmd);
 	}
 
-	utils_free_pointers(tmp, cmd, executable, locale_filename, NULL);
+	utils_free_pointers(4, tmp, cmd, executable, locale_filename, NULL);
 
 	if (result)
 		return working_dir;
@@ -732,8 +676,11 @@ static GPid build_run_cmd(GeanyDocument *doc)
 			g_free(utf8_working_dir);
 		}
 		if (! vte_send_cmd(vte_cmd))
+		{
 			ui_set_statusbar(FALSE,
 		_("Could not execute the file in the VTE because it probably contains a command."));
+			geany_debug("Could not execute the file in the VTE because it probably contains a command.");
+		}
 
 		/* show the VTE */
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(msgwindow.notebook), MSG_VTE);
@@ -839,7 +786,7 @@ static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
 			gchar *tmp;
 
 			color = (GPOINTER_TO_INT(data)) ? COLOR_DARK_RED : COLOR_BLACK;
-			g_strstrip(msg);
+			g_strchomp(msg);
 
 			if (build_parse_make_dir(msg, &tmp))
 			{
@@ -858,12 +805,12 @@ static gboolean build_iofunc(GIOChannel *ioc, GIOCondition cond, gpointer data)
 					GeanyDocument *doc = document_find_by_filename(filename);
 
 					if (doc)
-						editor_set_indicator_on_line(doc->editor, line - 1);
+						editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
 					color = COLOR_RED;	/* error message parsed on the line */
 				}
 				g_free(filename);
 			}
-			msgwin_compiler_add(color, msg);
+			msgwin_compiler_add_string(color, msg);
 
 			g_free(msg);
 		}
@@ -917,7 +864,7 @@ static void show_build_result_message(gboolean failure)
 	if (failure)
 	{
 		msg = _("Compilation failed.");
-		msgwin_compiler_add(COLOR_DARK_RED, msg);
+		msgwin_compiler_add_string(COLOR_DARK_RED, msg);
 		/* If msgwindow is hidden, user will want to display it to see the error */
 		if (! ui_prefs.msgwindow_visible)
 		{
@@ -931,7 +878,7 @@ static void show_build_result_message(gboolean failure)
 	else
 	{
 		msg = _("Compilation finished successfully.");
-		msgwin_compiler_add(COLOR_BLUE, msg);
+		msgwin_compiler_add_string(COLOR_BLUE, msg);
 		if (! ui_prefs.msgwindow_visible ||
 			gtk_notebook_get_current_page(GTK_NOTEBOOK(msgwindow.notebook)) != MSG_COMPILER)
 				ui_set_statusbar(FALSE, "%s", msg);
@@ -969,6 +916,7 @@ static void build_exit_cb(GPid child_pid, gint status, gpointer user_data)
 	build_info.pid = 0;
 	/* enable build items again */
 	build_menu_update(NULL);
+	ui_progress_bar_stop();
 }
 
 
@@ -1028,31 +976,24 @@ static void add_menu_accel(GeanyKeyGroup *group, guint kb_id,
 
 static void create_build_menu_gen(BuildMenuItems *menu_items)
 {
-	GtkWidget *menu, *item = NULL, *image, *separator;
+	GtkWidget *menu, *item = NULL, *separator;
 	GtkAccelGroup *accel_group = gtk_accel_group_new();
-	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
 	GeanyKeyGroup *group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_BUILD);
 
 	menu = gtk_menu_new();
 
 	/* compile the code */
-	item = gtk_image_menu_item_new_with_mnemonic(_("_Compile"));
+	item = ui_image_menu_item_new(GTK_STOCK_CONVERT, _("_Compile"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item, _("Compiles the current file"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_COMPILE, item);
-	image = gtk_image_new_from_stock("gtk-convert", GTK_ICON_SIZE_MENU);
-	gtk_widget_show(image);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
 	g_signal_connect(item, "activate", G_CALLBACK(on_build_compile_activate), NULL);
 	menu_items->item_compile = item;
 
 	/* build the code */
-	item = gtk_image_menu_item_new_with_mnemonic(_("_Build"));
+	item = ui_image_menu_item_new(GEANY_STOCK_BUILD, _("_Build"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item,
-				_("Builds the current file (generate an executable file)"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_LINK, item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_build_build_activate), NULL);
 	menu_items->item_link = item;
@@ -1065,8 +1006,6 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	item = gtk_image_menu_item_new_with_mnemonic(_("_Make All"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item, _("Builds the current file with the "
-										   "make tool and the default target"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_MAKE, item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_ALL));
@@ -1077,8 +1016,6 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	gtk_widget_show(item);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_MAKEOWNTARGET, item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item, _("Builds the current file with the "
-										   "make tool and the specified target"), NULL);
 	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_CUSTOM));
 	menu_items->item_make_custom = item;
@@ -1088,8 +1025,6 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	gtk_widget_show(item);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_MAKEOBJECT, item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item, _("Compiles the current file using the "
-										   "make tool"), NULL);
 	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_OBJECT));
 	menu_items->item_make_object = item;
@@ -1121,7 +1056,6 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	item = gtk_image_menu_item_new_from_stock("gtk-execute", accel_group);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item, _("Run or view the current file"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_RUN, item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_build_execute_activate), NULL);
 	menu_items->item_exec = item;
@@ -1132,16 +1066,10 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 	gtk_widget_set_sensitive(separator, FALSE);
 
 	/* arguments */
-	item = gtk_image_menu_item_new_with_mnemonic(_("_Set Includes and Arguments"));
+	item = ui_image_menu_item_new(GTK_STOCK_PREFERENCES, _("_Set Includes and Arguments"));
 	gtk_widget_show(item);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_OPTIONS, item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item,
-				_("Sets the includes and library paths for the compiler and "
-				  "the program arguments for execution"), NULL);
-	image = gtk_image_new_from_stock("gtk-preferences", GTK_ICON_SIZE_MENU);
-	gtk_widget_show(image);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
 	g_signal_connect(item, "activate", G_CALLBACK(on_build_arguments_activate), NULL);
 	menu_items->item_set_args = item;
 
@@ -1152,35 +1080,26 @@ static void create_build_menu_gen(BuildMenuItems *menu_items)
 
 static void create_build_menu_tex(BuildMenuItems *menu_items)
 {
-	GtkWidget *menu, *item, *image, *separator;
+	GtkWidget *menu, *item, *separator;
 	GtkAccelGroup *accel_group = gtk_accel_group_new();
-	GtkTooltips *tooltips = GTK_TOOLTIPS(lookup_widget(main_widgets.window, "tooltips"));
 	GeanyKeyGroup *group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_BUILD);
 
 	menu = gtk_menu_new();
 
 	/* DVI */
-	item = gtk_image_menu_item_new_with_mnemonic(_("LaTeX -> _DVI"));
+	item = ui_image_menu_item_new(GTK_STOCK_CONVERT, _("LaTeX -> _DVI"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item, _("Compiles the current file into a DVI file"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_COMPILE, item);
-	image = gtk_image_new_from_stock("gtk-convert", GTK_ICON_SIZE_MENU);
-	gtk_widget_show(image);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
 	g_signal_connect(item, "activate",
 				G_CALLBACK(on_build_tex_activate), GINT_TO_POINTER(LATEX_CMD_TO_DVI));
 	menu_items->item_compile = item;
 
 	/* PDF */
-	item = gtk_image_menu_item_new_with_mnemonic(_("LaTeX -> _PDF"));
+	item = ui_image_menu_item_new(GEANY_STOCK_BUILD, _("LaTeX -> _PDF"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item, _("Compiles the current file into a PDF file"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_LINK, item);
-	image = gtk_image_new_from_stock("gtk-convert", GTK_ICON_SIZE_MENU);
-	gtk_widget_show(image);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
 	g_signal_connect(item, "activate",
 				G_CALLBACK(on_build_tex_activate), GINT_TO_POINTER(LATEX_CMD_TO_PDF));
 	menu_items->item_link = item;
@@ -1193,8 +1112,6 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	item = gtk_image_menu_item_new_with_mnemonic(_("_Make All"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item, _("Builds the current file with the "
-										   "make tool and the default target"), NULL);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_MAKE, item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_ALL));
@@ -1205,8 +1122,6 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	gtk_widget_show(item);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_MAKEOWNTARGET, item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item, _("Builds the current file with the "
-										   "make tool and the specified target"), NULL);
 	g_signal_connect(item, "activate", G_CALLBACK(on_build_make_activate),
 		GINT_TO_POINTER(GBO_MAKE_CUSTOM));
 	menu_items->item_make_custom = item;
@@ -1236,27 +1151,19 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 
 	/* DVI view */
 #define LATEX_VIEW_DVI_LABEL _("_View DVI File") /* used later again */
-	item = gtk_image_menu_item_new_with_mnemonic(LATEX_VIEW_DVI_LABEL);
+	item = ui_image_menu_item_new(GTK_STOCK_FIND, LATEX_VIEW_DVI_LABEL);
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_RUN, item);
-	gtk_tooltips_set_tip(tooltips, item, _("Compile and view the current file"), NULL);
-	image = gtk_image_new_from_stock("gtk-find", GTK_ICON_SIZE_MENU);
-	gtk_widget_show(image);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
 	g_signal_connect(item, "activate",
 					G_CALLBACK(on_build_execute_activate), GINT_TO_POINTER(LATEX_CMD_VIEW_DVI));
 	menu_items->item_exec = item;
 
 	/* PDF view */
-	item = gtk_image_menu_item_new_with_mnemonic(_("V_iew PDF File"));
+	item = ui_image_menu_item_new(GTK_STOCK_FIND, _("V_iew PDF File"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_RUN2, item);
-	gtk_tooltips_set_tip(tooltips, item, _("Compile and view the current file"), NULL);
-	image = gtk_image_new_from_stock("gtk-find", GTK_ICON_SIZE_MENU);
-	gtk_widget_show(image);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
 	g_signal_connect(item, "activate",
 					G_CALLBACK(on_build_execute_activate), GINT_TO_POINTER(LATEX_CMD_VIEW_PDF));
 	menu_items->item_exec2 = item;
@@ -1268,15 +1175,10 @@ static void create_build_menu_tex(BuildMenuItems *menu_items)
 	gtk_widget_set_sensitive(separator, FALSE);
 
 	/* arguments */
-	item = gtk_image_menu_item_new_with_mnemonic(_("_Set Arguments"));
+	item = ui_image_menu_item_new(GTK_STOCK_PREFERENCES, _("_Set Arguments"));
 	gtk_widget_show(item);
 	GEANY_ADD_WIDGET_ACCEL(GEANY_KEYS_BUILD_OPTIONS, item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
-	gtk_tooltips_set_tip(tooltips, item,
-				_("Sets the program paths and arguments"), NULL);
-	image = gtk_image_new_from_stock("gtk-preferences", GTK_ICON_SIZE_MENU);
-	gtk_widget_show(image);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
 	g_signal_connect(item, "activate",
 		G_CALLBACK(on_build_arguments_activate), filetypes[GEANY_FILETYPES_LATEX]);
 	menu_items->item_set_args = item;
@@ -1302,7 +1204,7 @@ on_includes_arguments_tex_dialog_response  (GtkDialog *dialog,
 		struct build_programs *programs = ft->programs;
 
 		newstr = gtk_entry_get_text(
-				GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "tex_entry1")));
+				GTK_ENTRY(ui_lookup_widget(GTK_WIDGET(dialog), "tex_entry1")));
 		if (! utils_str_equal(newstr, programs->compiler))
 		{
 			if (programs->compiler) g_free(programs->compiler);
@@ -1310,7 +1212,7 @@ on_includes_arguments_tex_dialog_response  (GtkDialog *dialog,
 			programs->modified = TRUE;
 		}
 		newstr = gtk_entry_get_text(
-				GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "tex_entry2")));
+				GTK_ENTRY(ui_lookup_widget(GTK_WIDGET(dialog), "tex_entry2")));
 		if (! utils_str_equal(newstr, programs->linker))
 		{
 			if (programs->linker) g_free(programs->linker);
@@ -1318,7 +1220,7 @@ on_includes_arguments_tex_dialog_response  (GtkDialog *dialog,
 			programs->modified = TRUE;
 		}
 		newstr = gtk_entry_get_text(
-				GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "tex_entry3")));
+				GTK_ENTRY(ui_lookup_widget(GTK_WIDGET(dialog), "tex_entry3")));
 		if (! utils_str_equal(newstr, programs->run_cmd))
 		{
 			if (programs->run_cmd) g_free(programs->run_cmd);
@@ -1326,7 +1228,7 @@ on_includes_arguments_tex_dialog_response  (GtkDialog *dialog,
 			programs->modified = TRUE;
 		}
 		newstr = gtk_entry_get_text(
-				GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "tex_entry4")));
+				GTK_ENTRY(ui_lookup_widget(GTK_WIDGET(dialog), "tex_entry4")));
 		if (! utils_str_equal(newstr, programs->run_cmd2))
 		{
 			if (programs->run_cmd2) g_free(programs->run_cmd2);
@@ -1372,6 +1274,7 @@ static void show_includes_arguments_tex(void)
 			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
 
 		entries[0] = gtk_entry_new();
+		ui_entry_add_clear_icon(entries[0]);
 		gtk_entry_set_width_chars(GTK_ENTRY(entries[0]), 30);
 		if (ft->programs->compiler)
 		{
@@ -1379,7 +1282,7 @@ static void show_includes_arguments_tex(void)
 		}
 		gtk_table_attach_defaults(GTK_TABLE(table), entries[0], 1, 2, 0, 1);
 		g_object_set_data_full(G_OBJECT(dialog), "tex_entry1",
-					gtk_widget_ref(entries[0]), (GDestroyNotify)gtk_widget_unref);
+					g_object_ref(entries[0]), (GDestroyNotify)g_object_unref);
 	}
 
 	/* LaTeX -> PDF args */
@@ -1391,6 +1294,7 @@ static void show_includes_arguments_tex(void)
 			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
 
 		entries[1] = gtk_entry_new();
+		ui_entry_add_clear_icon(entries[1]);
 		gtk_entry_set_width_chars(GTK_ENTRY(entries[1]), 30);
 		if (ft->programs->linker)
 		{
@@ -1398,7 +1302,7 @@ static void show_includes_arguments_tex(void)
 		}
 		gtk_table_attach_defaults(GTK_TABLE(table), entries[1], 1, 2, 1, 2);
 		g_object_set_data_full(G_OBJECT(dialog), "tex_entry2",
-					gtk_widget_ref(entries[1]), (GDestroyNotify)gtk_widget_unref);
+					g_object_ref(entries[1]), (GDestroyNotify)g_object_unref);
 	}
 
 	/* View LaTeX -> DVI args */
@@ -1410,6 +1314,7 @@ static void show_includes_arguments_tex(void)
 			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
 
 		entries[2] = gtk_entry_new();
+		ui_entry_add_clear_icon(entries[2]);
 		gtk_entry_set_width_chars(GTK_ENTRY(entries[2]), 30);
 		if (ft->programs->run_cmd)
 		{
@@ -1417,7 +1322,7 @@ static void show_includes_arguments_tex(void)
 		}
 		gtk_table_attach_defaults(GTK_TABLE(table), entries[2], 1, 2, 2, 3);
 		g_object_set_data_full(G_OBJECT(dialog), "tex_entry3",
-					gtk_widget_ref(entries[2]), (GDestroyNotify)gtk_widget_unref);
+					g_object_ref(entries[2]), (GDestroyNotify)g_object_unref);
 	}
 
 	/* View LaTeX -> PDF args */
@@ -1429,6 +1334,7 @@ static void show_includes_arguments_tex(void)
 			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
 
 		entries[3] = gtk_entry_new();
+		ui_entry_add_clear_icon(entries[3]);
 		gtk_entry_set_width_chars(GTK_ENTRY(entries[3]), 30);
 		if (ft->programs->run_cmd2)
 		{
@@ -1436,7 +1342,7 @@ static void show_includes_arguments_tex(void)
 		}
 		gtk_table_attach_defaults(GTK_TABLE(table), entries[3], 1, 2, 3, 4);
 		g_object_set_data_full(G_OBJECT(dialog), "tex_entry4",
-					gtk_widget_ref(entries[3]), (GDestroyNotify)gtk_widget_unref);
+					g_object_ref(entries[3]), (GDestroyNotify)g_object_unref);
 	}
 
 	label = gtk_label_new(_("%f will be replaced by the current filename, e.g. test_file.c\n"
@@ -1471,7 +1377,7 @@ on_includes_arguments_dialog_response  (GtkDialog *dialog,
 		if (ft->actions->can_compile)
 		{
 			newstr = gtk_entry_get_text(
-					GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "includes_entry1")));
+					GTK_ENTRY(ui_lookup_widget(GTK_WIDGET(dialog), "includes_entry1")));
 			if (! utils_str_equal(newstr, programs->compiler))
 			{
 				if (programs->compiler) g_free(programs->compiler);
@@ -1482,7 +1388,7 @@ on_includes_arguments_dialog_response  (GtkDialog *dialog,
 		if (ft->actions->can_link)
 		{
 			newstr = gtk_entry_get_text(
-					GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "includes_entry2")));
+					GTK_ENTRY(ui_lookup_widget(GTK_WIDGET(dialog), "includes_entry2")));
 			if (! utils_str_equal(newstr, programs->linker))
 			{
 				if (programs->linker) g_free(programs->linker);
@@ -1493,7 +1399,7 @@ on_includes_arguments_dialog_response  (GtkDialog *dialog,
 		if (ft->actions->can_exec)
 		{
 			newstr = gtk_entry_get_text(
-					GTK_ENTRY(lookup_widget(GTK_WIDGET(dialog), "includes_entry3")));
+					GTK_ENTRY(ui_lookup_widget(GTK_WIDGET(dialog), "includes_entry3")));
 			if (! utils_str_equal(newstr, programs->run_cmd))
 			{
 				if (programs->run_cmd) g_free(programs->run_cmd);
@@ -1554,6 +1460,7 @@ static void show_includes_arguments_gen(void)
 			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
 
 		entries[0] = gtk_entry_new();
+		ui_entry_add_clear_icon(entries[0]);
 		gtk_entry_set_width_chars(GTK_ENTRY(entries[0]), 30);
 		if (ft->programs->compiler)
 		{
@@ -1563,7 +1470,7 @@ static void show_includes_arguments_gen(void)
 		row++;
 
 		g_object_set_data_full(G_OBJECT(dialog), "includes_entry1",
-					gtk_widget_ref(entries[0]), (GDestroyNotify)gtk_widget_unref);
+							   g_object_ref(entries[0]), (GDestroyNotify)g_object_unref);
 	}
 
 	/* lib-args */
@@ -1575,6 +1482,7 @@ static void show_includes_arguments_gen(void)
 			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
 
 		entries[1] = gtk_entry_new();
+		ui_entry_add_clear_icon(entries[1]);
 		gtk_entry_set_width_chars(GTK_ENTRY(entries[1]), 30);
 		if (ft->programs->linker)
 		{
@@ -1584,7 +1492,7 @@ static void show_includes_arguments_gen(void)
 		row++;
 
 		g_object_set_data_full(G_OBJECT(dialog), "includes_entry2",
-					gtk_widget_ref(entries[1]), (GDestroyNotify)gtk_widget_unref);
+							   g_object_ref(entries[1]), (GDestroyNotify)g_object_unref);
 	}
 
 	/* program-args */
@@ -1596,6 +1504,7 @@ static void show_includes_arguments_gen(void)
 			GTK_FILL, GTK_FILL | GTK_EXPAND, 6, 0);
 
 		entries[2] = gtk_entry_new();
+		ui_entry_add_clear_icon(entries[2]);
 		gtk_entry_set_width_chars(GTK_ENTRY(entries[2]), 30);
 		if (ft->programs->run_cmd)
 		{
@@ -1605,7 +1514,7 @@ static void show_includes_arguments_gen(void)
 		row++;
 
 		g_object_set_data_full(G_OBJECT(dialog), "includes_entry3",
-						gtk_widget_ref(entries[2]), (GDestroyNotify)gtk_widget_unref);
+							   g_object_ref(entries[2]), (GDestroyNotify)g_object_unref);
 
 		/* disable the run command if there is a valid project run command set */
 		if (app->project && NZV(app->project->run_cmd))
@@ -1657,20 +1566,24 @@ void build_menu_update(GeanyDocument *doc)
 	GeanyFiletype *ft;
 	gboolean have_path, can_build, can_make, can_run, can_stop, can_set_args, have_errors;
 	BuildMenuItems *menu_items;
+	static GtkWidget *menubar_build_menu = NULL;
 
+	if (menubar_build_menu == NULL) /* cache the build menu pointer */
+		menubar_build_menu = ui_lookup_widget(main_widgets.window, "menu_build1");
 	if (doc == NULL)
 		doc = document_get_current();
 	if (doc == NULL ||
 		(FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_NONE &&	doc->file_name == NULL))
 	{
-		gtk_widget_set_sensitive(lookup_widget(main_widgets.window, "menu_build1"), FALSE);
-		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(lookup_widget(main_widgets.window, "menu_build1")));
-		gtk_widget_set_sensitive(widgets.compile_button, FALSE);
-		gtk_widget_set_sensitive(widgets.run_button, FALSE);
+		gtk_widget_set_sensitive(menubar_build_menu, FALSE);
+		gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menubar_build_menu));
+		ui_widget_set_sensitive(widgets.compile_button, FALSE);
+		ui_widget_set_sensitive(widgets.build_button, FALSE);
+		ui_widget_set_sensitive(widgets.run_button, FALSE);
 		return;
 	}
 	else
-		gtk_widget_set_sensitive(lookup_widget(main_widgets.window, "menu_build1"), TRUE);
+		gtk_widget_set_sensitive(menubar_build_menu, TRUE);
 
 	ft = doc->file_type;
 	g_return_if_fail(ft != NULL);
@@ -1678,7 +1591,7 @@ void build_menu_update(GeanyDocument *doc)
 	menu_items = build_get_menu_items(ft->id);
 	/* Note: don't remove the submenu first because it can now cause an X hang if
 	 * the menu is already open when called from build_exit_cb(). */
-	gtk_menu_item_set_submenu(GTK_MENU_ITEM(lookup_widget(main_widgets.window, "menu_build1")),
+	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menubar_build_menu),
 		menu_items->menu);
 
 	have_path = (doc->file_name != NULL);
@@ -1725,8 +1638,22 @@ void build_menu_update(GeanyDocument *doc)
 	if (menu_items->item_set_args)
 		gtk_widget_set_sensitive(menu_items->item_set_args, can_set_args);
 
-	gtk_widget_set_sensitive(widgets.compile_button, can_build && ft->actions->can_compile);
-	gtk_widget_set_sensitive(widgets.run_button, can_run || can_stop);
+	if (widgets.toolitem_build != NULL)
+		gtk_widget_set_sensitive(widgets.toolitem_build,
+			can_build && ft->actions->can_link && ft->id != GEANY_FILETYPES_LATEX);
+	if (widgets.toolitem_make_all != NULL)
+		gtk_widget_set_sensitive(widgets.toolitem_make_all, can_make);
+	if (widgets.toolitem_make_custom != NULL)
+		gtk_widget_set_sensitive(widgets.toolitem_make_custom, can_make);
+	if (widgets.toolitem_make_object != NULL)
+		gtk_widget_set_sensitive(widgets.toolitem_make_object,
+			can_make && ft->id != GEANY_FILETYPES_LATEX);
+	if (widgets.toolitem_set_args != NULL)
+		gtk_widget_set_sensitive(widgets.toolitem_set_args, can_set_args);
+
+	ui_widget_set_sensitive(widgets.compile_button, can_build && ft->actions->can_compile);
+	ui_widget_set_sensitive(widgets.build_button, can_make);
+	ui_widget_set_sensitive(widgets.run_button, can_run || can_stop);
 
 	/* show the stop command if a program is running, otherwise show run command */
 	set_stop_button(can_stop);
@@ -1748,6 +1675,9 @@ static void set_stop_button(gboolean stop)
 	GtkWidget *menuitem =
 		build_get_menu_items(run_info.file_type_id)->item_exec;
 
+	if (widgets.run_button == NULL)
+		return;
+
 	if (stop && utils_str_equal(
 		gtk_tool_button_get_stock_id(GTK_TOOL_BUTTON(widgets.run_button)), "gtk-stop")) return;
 	if (! stop && utils_str_equal(
@@ -1876,7 +1806,7 @@ on_build_build_activate                (GtkMenuItem     *menuitem,
 {
 	GeanyDocument *doc = document_get_current();
 
-	if (doc == NULL)
+	if (doc == NULL || doc->file_type == NULL || ! doc->file_type->actions->can_link)
 		return;
 
 	if (doc->changed)
@@ -1889,6 +1819,16 @@ on_build_build_activate                (GtkMenuItem     *menuitem,
 }
 
 
+static void
+on_toolbutton_build_activate           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	last_toolbutton_action = GPOINTER_TO_INT(user_data);
+	g_object_set(widgets.build_action, "tooltip", _("Build the current file"), NULL);
+	on_build_build_activate(menuitem, user_data);
+}
+
+
 static void
 on_make_custom_input_response(const gchar *input)
 {
@@ -1949,6 +1889,32 @@ on_build_make_activate                 (GtkMenuItem     *menuitem,
 }
 
 
+static void
+on_toolbutton_make_activate            (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	gchar *msg;
+
+	last_toolbutton_action = GPOINTER_TO_INT(user_data);
+	switch (last_toolbutton_action)
+	{
+		case GBO_MAKE_ALL:
+			msg = _("Build the current file with Make and the default target");
+			break;
+		case GBO_MAKE_CUSTOM:
+			msg = _("Build the current file with Make and the specified target");
+			break;
+		case GBO_MAKE_OBJECT:
+			msg = _("Compile the current file with Make");
+			break;
+		default:
+			msg = NULL;
+	}
+	g_object_set(widgets.build_action, "tooltip", msg, NULL);
+	on_build_make_activate(menuitem, user_data);
+}
+
+
 static gboolean use_html_builtin(GeanyDocument *doc, GeanyFiletype *ft)
 {
 	gboolean use_builtin = FALSE;
@@ -1968,7 +1934,7 @@ static gboolean use_html_builtin(GeanyDocument *doc, GeanyFiletype *ft)
 	if (use_builtin)
 	{
 		gchar *uri = g_strconcat("file:///", g_path_skip_root(doc->file_name), NULL);
-		utils_start_browser(uri);
+		utils_open_browser(uri);
 		g_free(uri);
 
 		return TRUE;
@@ -2078,6 +2044,72 @@ on_build_previous_error                (GtkMenuItem     *menuitem,
 
 void build_init()
 {
-	widgets.compile_button = lookup_widget(main_widgets.window, "toolbutton_compile");
-	widgets.run_button = lookup_widget(main_widgets.window, "toolbutton_run");
+	GtkWidget *item;
+	GtkWidget *toolmenu;
+
+	widgets.build_action = toolbar_get_action_by_name("Build");
+	toolmenu = geany_menu_button_action_get_menu(GEANY_MENU_BUTTON_ACTION(widgets.build_action));
+
+	if (toolmenu != NULL)
+	{
+		/* build the code */
+		item = ui_image_menu_item_new(GEANY_STOCK_BUILD, _("_Build"));
+		gtk_widget_show(item);
+		gtk_container_add(GTK_CONTAINER(toolmenu), item);
+		g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_build_activate), NULL);
+		widgets.toolitem_build = item;
+
+		item = gtk_separator_menu_item_new();
+		gtk_widget_show(item);
+		gtk_container_add(GTK_CONTAINER(toolmenu), item);
+
+		/* build the code with make all */
+		item = gtk_image_menu_item_new_with_mnemonic(_("_Make All"));
+		gtk_widget_show(item);
+		gtk_container_add(GTK_CONTAINER(toolmenu), item);
+		g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_make_activate),
+			GINT_TO_POINTER(GBO_MAKE_ALL));
+		widgets.toolitem_make_all = item;
+
+		/* build the code with make custom */
+		item = gtk_image_menu_item_new_with_mnemonic(_("Make Custom _Target"));
+		gtk_widget_show(item);
+		gtk_container_add(GTK_CONTAINER(toolmenu), item);
+		g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_make_activate),
+			GINT_TO_POINTER(GBO_MAKE_CUSTOM));
+		widgets.toolitem_make_custom = item;
+
+		/* build the code with make object */
+		item = gtk_image_menu_item_new_with_mnemonic(_("Make _Object"));
+		gtk_widget_show(item);
+		gtk_container_add(GTK_CONTAINER(toolmenu), item);
+		g_signal_connect(item, "activate", G_CALLBACK(on_toolbutton_make_activate),
+			GINT_TO_POINTER(GBO_MAKE_OBJECT));
+		widgets.toolitem_make_object = item;
+
+		item = gtk_separator_menu_item_new();
+		gtk_widget_show(item);
+		gtk_container_add(GTK_CONTAINER(toolmenu), item);
+
+		/* arguments */
+		item = ui_image_menu_item_new(GTK_STOCK_PREFERENCES, _("_Set Includes and Arguments"));
+		gtk_widget_show(item);
+		gtk_container_add(GTK_CONTAINER(toolmenu), item);
+		g_signal_connect(item, "activate", G_CALLBACK(on_build_arguments_activate), NULL);
+		widgets.toolitem_set_args = item;
+	}
+
+	widgets.compile_button = toolbar_get_widget_by_name("Compile");
+	widgets.run_button = toolbar_get_widget_by_name("Run");
+	widgets.build_button = toolbar_get_widget_by_name("Build");
 }
+
+
+void build_toolbutton_build_clicked(GtkAction *action, gpointer user_data)
+{
+	if (last_toolbutton_action == GBO_BUILD)
+		on_build_build_activate(NULL, user_data);
+	else
+		on_build_make_activate(NULL, GINT_TO_POINTER(last_toolbutton_action));
+}
+
