@@ -1,8 +1,8 @@
 /*
  *      keyfile.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: keyfile.h 2738 2008-06-30 16:14:55Z eht16 $
+ * $Id: keyfile.h 3446 2009-01-04 18:30:42Z eht16 $
  */
 
 
@@ -26,6 +26,17 @@
 #define GEANY_KEYFILE_H 1
 
 
+extern GPtrArray *pref_groups;
+
+
+void configuration_init(void);
+
+void configuration_finalize(void);
+
+struct GeanyPrefGroup;
+
+void configuration_add_pref_group(struct GeanyPrefGroup *group, gboolean for_prefs_dialog);
+
 void configuration_save(void);
 
 gboolean configuration_load(void);
@@ -40,8 +51,6 @@ void configuration_load_session_files(GKeyFile *config);
 
 void configuration_save_session_files(GKeyFile *config);
 
-void configuration_read_filetype_extensions(void);
-
 /* set some settings which are already read from the config file, but need other things, like the
  * realisation of the main window */
 void configuration_apply_settings(void);
