@@ -1,8 +1,8 @@
 /*
  *      plugins.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugins.c 3095 2008-10-15 12:19:24Z ntrel $
+ * $Id: plugins.c 3567 2009-02-08 19:52:21Z eht16 $
  */
 
 /* Code to manage, load and unload plugins. */
@@ -49,7 +49,6 @@
 #include "dialogs.h"
 #include "msgwindow.h"
 #include "prefs.h"
-#include "geanyobject.h"
 #include "geanywraplabel.h"
 #include "build.h"
 #include "encodings.h"
@@ -58,6 +57,9 @@
 #include "keybindings.h"
 #include "navqueue.h"
 #include "main.h"
+#include "toolbar.h"
+#include "stash.h"
+#include "keyfile.h"
 
 
 #ifdef G_OS_WIN32
@@ -70,6 +72,7 @@
 typedef struct GeanyPluginPrivate
 {
 	GeanyAutoSeparator	toolbar_separator;
+	gboolean			resident;
 }
 GeanyPluginPrivate;
 
@@ -89,11 +92,14 @@ typedef struct Plugin
 
 	void		(*init) (GeanyData *data);			/* Called when the plugin is enabled */
 	GtkWidget*	(*configure) (GtkDialog *dialog);	/* plugin configure dialog, optional */
+	void		(*help) (void);					/* Called when the plugin should show some help, optional */
 	void		(*cleanup) (void);					/* Called when the plugin is disabled or when Geany exits */
 }
 Plugin;
 
 
+static gboolean want_plugins = FALSE;
+
 /* list of all available, loadable plugins, only valid as long as the plugin manager dialog is
  * opened, afterwards it will be destroyed */
 static GList *plugin_list = NULL;
@@ -106,10 +112,12 @@ static GtkWidget *menu_separator = NULL;
 static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data);
 
 void plugin_add_toolbar_item(GeanyPlugin *plugin, GtkToolItem *item);
+void plugin_module_make_resident(GeanyPlugin *plugin);
 
 
 static PluginFuncs plugin_funcs = {
-	&plugin_add_toolbar_item
+	&plugin_add_toolbar_item,
+	&plugin_module_make_resident
 };
 
 static DocumentFuncs doc_funcs = {
@@ -126,20 +134,34 @@ static DocumentFuncs doc_funcs = {
 	&document_set_encoding,
 	&document_set_text_changed,
 	&document_set_filetype,
-	&document_close
+	&document_close,
+	&document_index,
+	&document_save_file_as,
+	&document_rename_file,
+	&document_get_status_color
 };
 
 static EditorFuncs editor_funcs = {
-	&editor_set_indicator,
-	&editor_set_indicator_on_line,
-	&editor_clear_indicators,
 	&editor_get_indent_prefs,
-	&editor_create_widget
+	&editor_create_widget,
+	&editor_indicator_set_on_range,
+	&editor_indicator_set_on_line,
+	&editor_indicator_clear,
+	&editor_set_indent_type,
+	&editor_get_word_at_pos
 };
 
-static ScintillaFuncs sci_funcs = {
+static ScintillaFuncs scintilla_funcs = {
 	&scintilla_send_message,
-	&sci_cmd,
+	&scintilla_new
+};
+
+/* Macro to prevent a duplicate macro being generated in geanyfunctions.h */
+#define _scintilla_send_message_macro	scintilla_send_message
+
+static SciFuncs sci_funcs = {
+	&_scintilla_send_message_macro,
+	&sci_send_command,
 	&sci_start_undo_action,
 	&sci_end_undo_action,
 	&sci_set_text,
@@ -173,7 +195,8 @@ static ScintillaFuncs sci_funcs = {
 	&sci_get_current_line,
 	&sci_has_selection,
 	&sci_get_tab_width,
-	&sci_indicator_clear
+	&sci_indicator_clear,
+	&sci_indicator_set
 };
 
 static TemplateFuncs template_funcs = {
@@ -193,7 +216,11 @@ static UtilsFuncs utils_funcs = {
 	&utils_get_setting_integer,
 	&utils_get_setting_string,
 	&utils_spawn_sync,
-	&utils_spawn_async
+	&utils_spawn_async,
+	&utils_str_casecmp,
+	&utils_get_date_time,
+	&utils_open_browser,
+	&utils_string_replace_first
 };
 
 static UIUtilsFuncs uiutils_funcs = {
@@ -203,23 +230,34 @@ static UIUtilsFuncs uiutils_funcs = {
 	&ui_table_add_row,
 	&ui_path_box_new,
 	&ui_button_new_with_image,
-	&ui_add_document_sensitive
+	&ui_add_document_sensitive,
+	&ui_widget_set_tooltip_text,
+	&ui_image_menu_item_new,
+	&ui_lookup_widget,
+	&ui_progress_bar_start,
+	&ui_progress_bar_stop,
+	&ui_entry_add_clear_icon
 };
 
 static DialogFuncs dialog_funcs = {
 	&dialogs_show_question,
 	&dialogs_show_msgbox,
-	&dialogs_show_save_as
+	&dialogs_show_save_as,
+	&dialogs_show_input_numeric
 };
 
+/* Macro to prevent confusing macro being generated in geanyfunctions.h */
+#define _lookup_widget_macro	ui_lookup_widget
+
+/* deprecated */
 static SupportFuncs support_funcs = {
-	&lookup_widget
+	&_lookup_widget_macro
 };
 
 static MsgWinFuncs msgwin_funcs = {
 	&msgwin_status_add,
-	&msgwin_compiler_add_fmt,
-	&msgwin_msg_add_fmt,
+	&msgwin_compiler_add,
+	&msgwin_msg_add,
 	&msgwin_clear_tab,
 	&msgwin_switch_tab
 };
@@ -252,10 +290,10 @@ static HighlightingFuncs highlighting_funcs = {
 	&highlighting_get_style
 };
 
-
 static FiletypeFuncs filetype_funcs = {
 	&filetypes_detect_from_file,
-	&filetypes_lookup_by_name
+	&filetypes_lookup_by_name,
+	&filetypes_index
 };
 
 static NavQueueFuncs navqueue_funcs = {
@@ -263,7 +301,8 @@ static NavQueueFuncs navqueue_funcs = {
 };
 
 static MainFuncs main_funcs = {
-	&main_reload_configuration
+	&main_reload_configuration,
+	&main_locale_init
 };
 
 static GeanyFunctions geany_functions = {
@@ -284,7 +323,9 @@ static GeanyFunctions geany_functions = {
 	&navqueue_funcs,
 	&editor_funcs,
 	&main_funcs,
-	&plugin_funcs
+	&plugin_funcs,
+	&scintilla_funcs,
+	&msgwin_funcs
 };
 
 static GeanyData geany_data;
@@ -501,8 +542,12 @@ plugin_init(Plugin *plugin)
 	g_return_if_fail(plugin->init);
 	plugin->init(&geany_data);
 
+	if (p_geany_plugin && (*p_geany_plugin)->priv->resident)
+		g_module_make_resident(plugin->module);
+
 	/* store some function pointers for later use */
 	g_module_symbol(plugin->module, "plugin_configure", (void *) &plugin->configure);
+	g_module_symbol(plugin->module, "plugin_help", (void *) &plugin->help);
 	g_module_symbol(plugin->module, "plugin_cleanup", (void *) &plugin->cleanup);
 	if (plugin->cleanup == NULL)
 	{
@@ -708,7 +753,7 @@ plugin_free(Plugin *plugin)
 
 /* load active plugins at startup */
 static void
-load_active_plugins()
+load_active_plugins(void)
 {
 	guint i, len;
 
@@ -739,7 +784,7 @@ load_plugins_from_path(const gchar *path)
 	for (item = list; item != NULL; item = g_slist_next(item))
 	{
 		tmp = strrchr(item->data, '.');
-		if (tmp == NULL || strcasecmp(tmp, "." PLUGIN_EXT) != 0)
+		if (tmp == NULL || utils_str_casecmp(tmp, "." PLUGIN_EXT) != 0)
 			continue;
 
 		fname = g_strconcat(path, G_DIR_SEPARATOR_S, item->data, NULL);
@@ -755,7 +800,7 @@ load_plugins_from_path(const gchar *path)
 #ifdef G_OS_WIN32
 static gchar *get_plugin_path()
 {
-	gchar *install_dir = g_win32_get_package_installation_directory("geany", NULL);
+	gchar *install_dir = g_win32_get_package_installation_directory(NULL, NULL);
 	gchar *path;
 
 	path = g_strconcat(install_dir, "\\lib", NULL);
@@ -771,7 +816,7 @@ static void load_all_plugins(void)
 	gchar *path;
 
 	path = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "plugins", NULL);
-	/* first load plugins in ~/.geany/plugins/, then in $prefix/lib/geany */
+	/* first load plugins in ~/.config/geany/plugins/, then in $prefix/lib/geany */
 	load_plugins_from_path(path);
 	g_free(path);
 #ifdef G_OS_WIN32
@@ -809,12 +854,14 @@ static void on_tools_menu_show(GtkWidget *menu_item, G_GNUC_UNUSED gpointer user
 }
 
 
-void plugins_init()
+/* Calling this starts up plugin support */
+void plugins_load_active(void)
 {
 	GtkWidget *widget;
 
+	want_plugins = TRUE;
+
 	geany_data_init();
-	geany_object = geany_object_new();
 
 	widget = gtk_separator_menu_item_new();
 	gtk_widget_show(widget);
@@ -833,6 +880,15 @@ void plugins_init()
 }
 
 
+static gint cmp_plugin_names(gconstpointer a, gconstpointer b)
+{
+	const Plugin *pa = a;
+	const Plugin *pb = b;
+
+	return strcmp(pa->info.name, pb->info.name);
+}
+
+
 static void update_active_plugins_pref(void)
 {
 	gint i = 0;
@@ -847,6 +903,10 @@ static void update_active_plugins_pref(void)
 		return;
 	}
 
+	/* sort the list so next time tools menu items are sorted by plugin name
+	 * (not ideal to do here, but better than nothing) */
+	active_plugin_list = g_list_sort(active_plugin_list, cmp_plugin_names);
+
 	active_plugins_pref = g_new0(gchar*, count + 1);
 
 	for (list = g_list_first(active_plugin_list); list != NULL; list = list->next)
@@ -867,31 +927,32 @@ static void update_active_plugins_pref(void)
 }
 
 
-void plugins_save_prefs(GKeyFile *config)
+static void on_save_settings(GKeyFile *config)
 {
-	g_key_file_set_boolean(config, "plugins", "load_plugins", prefs.load_plugins);
-
-	update_active_plugins_pref();
-	if (active_plugins_pref != NULL)
-		g_key_file_set_string_list(config, "plugins", "active_plugins",
-			(const gchar**)active_plugins_pref, g_strv_length(active_plugins_pref));
-	else
-	{
-		/* use an empty dummy array to override maybe exisiting value */
-		const gchar *dummy[] = { "" };
-		g_key_file_set_string_list(config, "plugins", "active_plugins", dummy, 1);
-	}
+	/* if plugins are disabled, don't clear list of active plugins */
+	if (want_plugins)
+		update_active_plugins_pref();
 }
 
 
-void plugins_load_prefs(GKeyFile *config)
+/* called even if plugin support is disabled */
+void plugins_init(void)
 {
-	prefs.load_plugins = utils_get_setting_boolean(config, "plugins", "load_plugins", TRUE);
-	active_plugins_pref = g_key_file_get_string_list(config, "plugins", "active_plugins", NULL, NULL);
+	GeanyPrefGroup *group;
+
+	group = stash_group_new("plugins");
+	configuration_add_pref_group(group, TRUE);
+
+	stash_group_add_toggle_button(group, &prefs.load_plugins,
+		"load_plugins", TRUE, "check_plugins");
+
+	g_signal_connect(geany_object, "save-settings", G_CALLBACK(on_save_settings), NULL);
+	stash_group_add_string_vector(group, &active_plugins_pref, "active_plugins", NULL);
 }
 
 
-void plugins_free(void)
+/* called even if plugin support is disabled */
+void plugins_finalize(void)
 {
 	if (failed_plugins_list != NULL)
 	{
@@ -904,9 +965,6 @@ void plugins_free(void)
 		g_list_free(active_plugin_list);
 	}
 	g_strfreev(active_plugins_pref);
-
-	g_object_unref(geany_object);
-	geany_object = NULL; /* to mark the object as invalid for any code which tries to emit signals */
 }
 
 
@@ -918,7 +976,9 @@ enum
 	PLUGIN_COLUMN_NAME,
 	PLUGIN_COLUMN_FILE,
 	PLUGIN_COLUMN_PLUGIN,
-	PLUGIN_N_COLUMNS
+	PLUGIN_N_COLUMNS,
+	PM_BUTTON_CONFIGURE,
+	PM_BUTTON_HELP
 };
 
 typedef struct
@@ -928,6 +988,7 @@ typedef struct
 	GtkListStore *store;
 	GtkWidget *description_label;
 	GtkWidget *configure_button;
+	GtkWidget *help_button;
 }
 PluginManagerWidgets;
 
@@ -948,6 +1009,7 @@ void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data)
 		{
 			gchar *text;
 			PluginInfo *pi;
+			gboolean is_active;
 
 			pi = &p->info;
 			text = g_strdup_printf(
@@ -957,8 +1019,9 @@ void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data)
 			geany_wrap_label_set_text(GTK_LABEL(pm_widgets.description_label), text);
 			g_free(text);
 
-			gtk_widget_set_sensitive(pm_widgets.configure_button,
-				p->configure != NULL && g_list_find(active_plugin_list, p) != NULL);
+			is_active = is_active_plugin(p);
+			gtk_widget_set_sensitive(pm_widgets.configure_button, p->configure != NULL && is_active);
+			gtk_widget_set_sensitive(pm_widgets.help_button, p->help != NULL && is_active);
 		}
 	}
 }
@@ -967,6 +1030,7 @@ void pm_selection_changed(GtkTreeSelection *selection, gpointer user_data)
 static void pm_plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer data)
 {
 	gboolean old_state, state;
+	gboolean is_active;
 	gchar *file_name;
 	GtkTreeIter iter;
 	GtkTreePath *path = gtk_tree_path_new_from_string(pth);
@@ -986,7 +1050,9 @@ static void pm_plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer
 
 	/* unload plugin module */
 	if (!state)
-		keybindings_write_to_file();	/* save shortcuts (only need this group, but it doesn't take long) */
+		/* save shortcuts (only need this group, but it doesn't take long) */
+		keybindings_write_to_file();
+
 	plugin_free(p);
 
 	/* reload plugin module and initialize it if item is checked */
@@ -1001,9 +1067,10 @@ static void pm_plugin_toggled(GtkCellRendererToggle *cell, gchar *pth, gpointer
 
 	g_free(file_name);
 
-	/* set again the sensitiveness of the configure button */
-	gtk_widget_set_sensitive(pm_widgets.configure_button,
-		p->configure != NULL && is_active_plugin(p));
+	/* set again the sensitiveness of the configure and help buttons */
+	is_active = is_active_plugin(p);
+	gtk_widget_set_sensitive(pm_widgets.configure_button, p->configure != NULL && is_active);
+	gtk_widget_set_sensitive(pm_widgets.help_button, p->help != NULL && is_active);
 }
 
 
@@ -1103,7 +1170,7 @@ static void configure_plugin(Plugin *p)
 }
 
 
-void pm_on_configure_button_clicked(GtkButton *button, gpointer user_data)
+void pm_on_plugin_button_clicked(GtkButton *button, gpointer user_data)
 {
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
@@ -1117,7 +1184,10 @@ void pm_on_configure_button_clicked(GtkButton *button, gpointer user_data)
 
 		if (p != NULL)
 		{
-			configure_plugin(p);
+			if (GPOINTER_TO_INT(user_data) == PM_BUTTON_CONFIGURE)
+				configure_plugin(p);
+			else if (GPOINTER_TO_INT(user_data) == PM_BUTTON_HELP && p->help != NULL)
+				p->help();
 		}
 	}
 }
@@ -1182,7 +1252,12 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 	pm_widgets.configure_button = gtk_button_new_from_stock(GTK_STOCK_PREFERENCES);
 	gtk_widget_set_sensitive(pm_widgets.configure_button, FALSE);
 	g_signal_connect(pm_widgets.configure_button, "clicked",
-		G_CALLBACK(pm_on_configure_button_clicked), NULL);
+		G_CALLBACK(pm_on_plugin_button_clicked), GINT_TO_POINTER(PM_BUTTON_CONFIGURE));
+
+	pm_widgets.help_button = gtk_button_new_from_stock(GTK_STOCK_HELP);
+	gtk_widget_set_sensitive(pm_widgets.help_button, FALSE);
+	g_signal_connect(pm_widgets.help_button, "clicked",
+		G_CALLBACK(pm_on_plugin_button_clicked), GINT_TO_POINTER(PM_BUTTON_HELP));
 
 	label2 = gtk_label_new(_("<b>Plugin details:</b>"));
 	gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
@@ -1197,13 +1272,14 @@ static void pm_show_dialog(GtkMenuItem *menuitem, gpointer user_data)
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), label2, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), pm_widgets.help_button, FALSE, FALSE, 4);
 	gtk_box_pack_start(GTK_BOX(hbox), pm_widgets.configure_button, FALSE, FALSE, 0);
 
-	label_vbox = gtk_vbox_new(FALSE, 0);
+	label_vbox = gtk_vbox_new(FALSE, 3);
 	gtk_box_pack_start(GTK_BOX(label_vbox), hbox, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(label_vbox), desc_win, FALSE, FALSE, 0);
 
-	vbox2 = gtk_vbox_new(FALSE, 6);
+	vbox2 = gtk_vbox_new(FALSE, 3);
 	gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(vbox2), swin, TRUE, TRUE, 0);
 	gtk_box_pack_start(GTK_BOX(vbox2), label_vbox, FALSE, FALSE, 0);
@@ -1234,14 +1310,13 @@ void plugin_add_toolbar_item(GeanyPlugin *plugin, GtkToolItem *item)
 	{
 		GtkToolItem *sep;
 
-		pos = ui_get_toolbar_insert_position();
-		/* pos should be valid even if the quit btn is hidden */
-		g_return_if_fail(pos >= 0);
-		gtk_toolbar_insert(toolbar, item, pos);
+		pos = toolbar_get_insert_position();
 
 		sep = gtk_separator_tool_item_new();
-		gtk_toolbar_insert(toolbar, sep, pos + 1);
+		gtk_toolbar_insert(toolbar, sep, pos);
 		autosep->widget = GTK_WIDGET(sep);
+
+		gtk_toolbar_insert(toolbar, item, pos + 1);
 	}
 	else
 	{
@@ -1254,4 +1329,20 @@ void plugin_add_toolbar_item(GeanyPlugin *plugin, GtkToolItem *item)
 }
 
 
+/** Ensures that a plugin's module (*.so) will never be unloaded.
+ *  This is necessary if you register new GTypes in your plugin, e.g. when using own classes
+ *  using the GObject system.
+ *
+ * @param plugin Must be @ref geany_plugin.
+ *
+ *  @since 0.16
+ */
+void plugin_module_make_resident(GeanyPlugin *plugin)
+{
+	g_return_if_fail(plugin);
+
+	plugin->priv->resident = TRUE;
+}
+
+
 #endif
