@@ -1,8 +1,9 @@
 /*
  *      socket.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006 Hiroyuki Yamamoto (author of Sylpheed)
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: socket.c 2287 2008-02-27 13:17:29Z eht16 $
+ * $Id: socket.c 3503 2009-01-22 20:32:04Z eht16 $
  */
 
 /*
@@ -38,8 +39,10 @@
  * .\n
  * The first thing should be the command name followed by the data belonging to the command and
  * to mark the end of data send a single '.'. Each message should be ended with \n.
+ * The command window is only available on Windows and takes no additional data, instead it
+ * writes back a Windows handle (HWND) for the main window to set it to the foreground (focus).
  *
- * At the moment the commands open, line and column are available.
+ * At the moment the commands window, open, line and column are available.
  *
  * About the socket files on Unix-like systems:
  * Geany creates a socket in /tmp (or any other directory returned by g_get_tmp_dir()) and
@@ -64,6 +67,8 @@
 # include <netinet/in.h>
 # include <glib/gstdio.h>
 #else
+# include <gdk/gdkwin32.h>
+# include <windows.h>
 # include <winsock2.h>
 # include <ws2tcpip.h>
 #endif
@@ -77,15 +82,14 @@
 #include "support.h"
 #include "ui_utils.h"
 #include "utils.h"
+#include "encodings.h"
 
 
 
 #ifdef G_OS_WIN32
 #define REMOTE_CMD_PORT		49876
-#define SockDesc			SOCKET
 #define SOCKET_IS_VALID(s)	((s) != INVALID_SOCKET)
 #else
-#define SockDesc			gint
 #define SOCKET_IS_VALID(s)	((s) >= 0)
 #define INVALID_SOCKET		(-1)
 #endif
@@ -141,9 +145,9 @@ void send_open_command(gint sock, gint argc, gchar **argv)
 
 	socket_fd_write_all(sock, "open\n", 5);
 
-	for(i = 1; i < argc && argv[i] != NULL; i++)
+	for (i = 1; i < argc && argv[i] != NULL; i++)
 	{
-		filename = get_argv_filename(argv[i]);
+		filename = main_get_argv_filename(argv[i]);
 
 		/* if the filename is valid or if a new file should be opened is check on the other side */
 		if (filename != NULL)
@@ -190,16 +194,11 @@ static void remove_socket_link_full(void)
 gint socket_init(gint argc, gchar **argv)
 {
 	gint sock;
-
 #ifdef G_OS_WIN32
 	HANDLE hmutex;
-	/* we need a mutex name which is unique for the used configuration dir,
-	 * but we can't use the whole path, so build a hash on the configdir and use it */
-	gchar *mutex_name = g_strdup_printf("Geany%d", g_str_hash(app->configdir));
-
+	HWND hwnd;
 	socket_init_win32();
-	hmutex = CreateMutexA(NULL, FALSE, mutex_name);
-	g_free(mutex_name);
+	hmutex = CreateMutexA(NULL, FALSE, "Geany");
 	if (! hmutex)
 	{
 		geany_debug("cannot create Mutex\n");
@@ -207,6 +206,12 @@ gint socket_init(gint argc, gchar **argv)
 	}
 	if (GetLastError() != ERROR_ALREADY_EXISTS)
 	{
+		/* To support multiple instances with different configuration directories (as we do on
+		 * non-Windows systems) we would need to use different port number s but it might be
+		 * difficult to get a port number which is unique for a configuration directory (path)
+		 * and which is unused. This port number has to be guessed by the first and new instance
+		 * and the only data is the configuration directory path.
+		 * For now we use one port number, that is we support only one instance at all. */
 		sock = socket_fd_open_inet(REMOTE_CMD_PORT);
 		if (sock < 0)
 			return 0;
@@ -246,6 +251,14 @@ gint socket_init(gint argc, gchar **argv)
 #endif
 
 	/* remote command mode, here we have another running instance and want to use it */
+
+#ifdef G_OS_WIN32
+	/* first we send a request to retrieve the window handle and focus the window */
+	socket_fd_write_all(sock, "window\n", 7);
+	if (socket_fd_read(sock, (gchar *)&hwnd, sizeof(hwnd)) == sizeof(hwnd))
+		SetForegroundWindow(hwnd);
+#endif
+	/* now we send the command line args */
 	if (argc > 1)
 	{
 		send_open_command(sock, argc, argv);
@@ -341,12 +354,12 @@ static gint socket_fd_open_unix(const gchar *path)
 		g_get_tmp_dir(), G_DIR_SEPARATOR, g_random_int());
 
 	if (utils_is_file_writeable(real_path) != 0)
-	{	/* if real_path is not writable for us, fall back to ~/.geany/geany_socket_*_* */
+	{	/* if real_path is not writable for us, fall back to ~/.config/geany/geany_socket_*_* */
 		/* instead of creating a symlink and print a warning */
 		g_warning("Socket %s could not be written, using %s as fallback.", real_path, path);
 		setptr(real_path, g_strdup(path));
 	}
-	/* create a symlink in e.g. ~/.geany/geany_socket_hostname__0 to /tmp/geany_socket.499602d2 */
+	/* create a symlink in e.g. ~/.config/geany/geany_socket_hostname__0 to /tmp/geany_socket.499602d2 */
 	else if (symlink(real_path, path) != 0)
 	{
 		perror("symlink");
@@ -391,7 +404,7 @@ static gint socket_fd_close(gint fd)
 #ifdef G_OS_WIN32
 static gint socket_fd_open_inet(gushort port)
 {
-	SockDesc sock;
+	SOCKET sock;
 	struct sockaddr_in addr;
 	gchar val;
 
@@ -435,7 +448,7 @@ static gint socket_fd_open_inet(gushort port)
 
 static gint socket_fd_connect_inet(gushort port)
 {
-	SockDesc sock;
+	SOCKET sock;
 	struct sockaddr_in addr;
 
 	sock = socket(AF_INET, SOCK_STREAM, 0);
@@ -472,14 +485,31 @@ static void socket_init_win32(void)
 #endif
 
 
+static void handle_input_filename(const gchar *buf)
+{
+	gchar *utf8_filename, *locale_filename;
+
+	/* we never know how the input is encoded, so do the best auto detection we can */
+	if (! g_utf8_validate(buf, -1, NULL))
+		utf8_filename = encodings_convert_to_utf8(buf, (gsize) -1, NULL);
+	else
+		utf8_filename = g_strdup(buf);
+
+	locale_filename = utils_get_locale_from_utf8(utf8_filename);
+	g_free(utf8_filename);
+	if (locale_filename)
+		main_handle_filename(locale_filename);
+	g_free(locale_filename);
+}
+
+
 gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpointer data)
 {
 	gint fd, sock;
 	gchar buf[4096];
 	struct sockaddr_in caddr;
-	guint caddr_len;
-
-	caddr_len = sizeof(caddr);
+	guint caddr_len = sizeof(caddr);
+	GtkWidget *window = data;
 
 	fd = g_io_channel_unix_get_fd(source);
 	sock = accept(fd, (struct sockaddr *)&caddr, &caddr_len);
@@ -489,31 +519,10 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 	{
 		if (strncmp(buf, "open", 4) == 0)
 		{
-			document_delay_colourise();
-
 			while (socket_fd_gets(sock, buf, sizeof(buf)) != -1 && *buf != '.')
 			{
-				g_strstrip(buf); /* remove \n char */
-
-				if (g_file_test(buf, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
-					document_open_file(buf, FALSE, NULL, NULL);
-				else
-				{	/* create new file if it doesn't exist */
-					gint idx;
-
-					idx = document_new_file(buf, NULL, NULL);
-					if (DOC_IDX_VALID(idx))
-						ui_add_recent_file(doc_list[idx].file_name);
-					else
-						geany_debug("got data from socket, but it does not look like a filename");
-				}
+				handle_input_filename(g_strstrip(buf));
 			}
-			document_colourise_new();
-
-			gtk_window_deiconify(GTK_WINDOW(app->window));
-#ifdef G_OS_WIN32
-			gtk_window_present(GTK_WINDOW(app->window));
-#endif
 		}
 		else if (strncmp(buf, "line", 4) == 0)
 		{
@@ -533,7 +542,19 @@ gboolean socket_lock_input_cb(GIOChannel *source, GIOCondition condition, gpoint
 				cl_options.goto_column = atoi(buf);
 			}
 		}
+#ifdef G_OS_WIN32
+		else if (strncmp(buf, "window", 6) == 0)
+		{
+			HWND hwnd = (HWND) gdk_win32_drawable_get_handle(GDK_DRAWABLE(window->window));
+			socket_fd_write(sock, (gchar *)&hwnd, sizeof(hwnd));
+		}
+#endif
 	}
+	gtk_window_present(GTK_WINDOW(window));
+#ifdef G_OS_WIN32
+	gdk_window_show(window->window);
+#endif
+
 	socket_fd_close(sock);
 
 	return TRUE;
@@ -599,14 +620,12 @@ static gint socket_fd_check_io(gint fd, GIOCondition cond)
 
 #ifdef G_OS_UNIX
 	/* checking for non-blocking mode */
-
 	flags = fcntl(fd, F_GETFL, 0);
 	if (flags < 0)
 	{
 		perror("fcntl");
 		return 0;
 	}
-
 	if ((flags & O_NONBLOCK) != 0)
 		return 0;
 #endif
@@ -629,7 +648,7 @@ static gint socket_fd_check_io(gint fd, GIOCondition cond)
 	}
 	else
 	{
-		geany_debug("Socket IO timeout\n");
+		geany_debug("Socket IO timeout");
 		return -1;
 	}
 }
@@ -665,7 +684,6 @@ gint socket_fd_write(gint fd, const gchar *buf, gint len)
 #endif
 }
 
-
 #endif
 
 
