@@ -1,8 +1,8 @@
 /*
  *      win32.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: win32.c 3097 2008-10-15 14:14:31Z eht16 $
+ *  $Id: win32.c 3446 2009-01-04 18:30:42Z eht16 $
  */
 
 /*
@@ -716,33 +716,6 @@ void win32_init_debug_code()
 }
 
 
-/* Used to get special Windows folder paths like %appdata%,
- * Code taken from Sylpheed, thanks */
-gchar *win32_get_appdata_folder()
-{
-	gchar *folder = NULL;
-	gint nfolder = CSIDL_APPDATA;
-	HRESULT hr;
-
-	if (G_WIN32_HAVE_WIDECHAR_API())
-	{
-		wchar_t path[MAX_PATH + 1];
-		hr = SHGetFolderPathW(NULL, nfolder, NULL, 0, path);
-		if (hr == S_OK)
-			folder = g_utf16_to_utf8(path, -1, NULL, NULL, NULL);
-	}
-	else
-	{
-		gchar path[MAX_PATH + 1];
-		hr = SHGetFolderPathA(NULL, nfolder, NULL, 0, path);
-		if (hr == S_OK)
-			folder = g_locale_to_utf8(path, -1, NULL, NULL, NULL);
-	}
-
-	return folder;
-}
-
-
 gchar *win32_get_hostname()
 {
 	gchar hostname[100];
